###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Erik Enge <erik@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.2 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

from Globals import HTMLFile
import DateTime
from ZPublisher.HTTPRequest import html_quote
import Products
import ZODB
import string
import property_manager
import object_manager
import html_widgets

class base_item(property_manager.property_manager,
		object_manager.object_manager):
	"""Base class we use for easy access to often used methods."""

	meta_types = ()
	help_name = ''

	# How are we to go about handling permissions derived from the
	# base classes?

	__ac_permissions__ = () # !FIXME!

	base_delete = HTMLFile('dtml/base_delete', globals())
	base_edit = HTMLFile('dtml/base_edit', globals())
	base_edit_display = HTMLFile('dtml/base_edit_display', globals())
	base_display = HTMLFile('dtml/base_display', globals())
	index_html = HTMLFile('dtml/base_index', globals())

	search_interface = HTMLFile('dtml/base_search_interface', globals())
	search_results = HTMLFile('dtml/base_search_results', globals())

	local_frame = HTMLFile('dtml/base_local_frame', globals())

	use_local_navigation = 1
	use_local_preferences = 1
	use_breadcrumbs = 1
	index_sheet = ()
	search_sheets = ()
	creator = 'Unknown'

	def get_object_sheet(self, meta_type):
		"""Returns the specified object sheet."""

		try:

			return self.get_class_info_for_meta_type(meta_type)['object_sheet']

		except KeyError:

			return (
					('Title', 'get_navigation_title()',),
					('Date created', 'render_date(object.date_created)'),
					('Creator', 'creator'),
			)


	def get_object_sheet_heading(self, meta_type):
		"""Returns the listing header data for the given object."""

		headers = map(lambda x: x[0], self.get_object_sheet(meta_type))

		return {'size':len(headers), 'headers':headers}


	def get_class_info_for_meta_type(self, meta_type):
		"""Returns the class info for the given meta type."""

		for product in Products.product_registry.items():

			try:

				return product[-1][meta_type]

			except KeyError:

				pass

		raise KeyError

			
	def get_navigation_meta_types(self):
		"""Returns relevant meta types for navigation."""

		return self.meta_types


	def has_property(self, prop_name):
		"Return true if self._properties contains prop_name."
		for x in range(len(self._properties)): 
			if self._properties[x]['id'] == prop_name:
				return 1
			'''
			for key in self._properties[x].keys():
				if str(prop_name) == str(self._properties[x]['id']):
					return 1
			'''
		return 0

	def name_title_or_id(self):
		"Returns either job_title, firstname + surname, name, title, meta_type or id - in that order."
		if self.has_property('job_title') == 1:
			job_title=self.get_job_title()
		else:
			job_title=''
		
		if self.has_property('name') == 1:
			name=self.name
		else:
			name=''

		if self.has_property('firstname') == 1:
			firstname=self.firstname
			surname=self.surname
		else:
			firstname=''
			surname=''

		title=self.title
		if callable(title):
			title=title()

		id=self.id
		if callable(id):
			id=id()

		if (len(job_title) > 0 and self.meta_type != 'Candidate'):
			return job_title +' / '+str(self.id)
		elif len(firstname) > 0:
			return firstname +' '+ surname
		elif len(name) > 0:
			return name
		elif len(title) > 0:
			return title
		else:
			return id


	get_navigation_title = name_title_or_id

	def get_help_html(self):
		"""Returns a submission button used for accessing help."""

		if self.help_name:

			return """<input type="button" onclick="window.open('%s');" value="Help">""" % \
				("/Control_Panel/Products/%s/Help/%s" % (self.product_name, self.help_name))

		else:

			return ""


	def local_navigation(self, *args, **kw):
		"""Returns a set of widgets for adding objects."""

		if self.REQUEST.has_key('skip_local_navigation') and\
			self.REQUEST['skip_local_navigation']:

			return ""

		if string.find(self.REQUEST.URL0, 'manage_add') > -1 or \
			 string.find(self.REQUEST.URL0, 'manage_edit') > -1:

			return ""

		actions = []
		meta_types = list(self.get_navigation_meta_types())

		if getattr(self.aq_base, 'use_all_meta_types', None):

			meta_types.extend(list(self.all_meta_types()))

		for subclass in meta_types:

			actions.append("""<option value="%s">%s</option>""" % \
						(subclass['action'], subclass['name']))

		else:

			if actions:

				action_html = """<select name=":method">%s</select>""" % \
					string.join(actions)
				action_html = action_html + """<input type="submit" value=" Add ">"""

			else:

				action_html = ""

		if getattr(self.aq_base, 'deleteable_object', None):

			delete_html = """<input type="submit" value=" Delete " name="delete:method">"""

		else:

			delete_html = ""

		if getattr(self.aq_base, 'editable_object', None):

			edit_html = """<input type="submit" value=" Edit " name="base_edit:method">"""

		else:

			edit_html = ""

		help_html = self.get_help_html()


		return """<form action="%s" target="main_frame">""" % self.relative_url() + \
			action_html + delete_html + edit_html + help_html + """</form>"""


	def render_modification_time(self):
		"""Returns a local display of the modification time."""

		return self.render_date(self.bobobase_modification_time)


	def copy(self):
		"""Returns a copy of self."""

		return self.getParentNode().manage_copyObjects(self.id)


	def paste(self, copy_data, new_id=None):
		"""Pastes the copy into self."""

		self.manage_pasteObjects(cb_copy_data=copy_data)


	def cut(self):
		"""Cuts self from containing object."""

		return self.getParentNode().manage_cutObjects(self.id)


	def delete(self, RESPONSE=None):
		"""Deletes self."""

		self.getParentNode().manage_delObjects(self.id)

		if RESPONSE is not None:

			RESPONSE.redirect(self.getParentNode().relative_url())


	def navigation(self, separator_html=' &gt;&gt; ',):
		"""Renders breadcrumb navigation."""

		# FIXME
		# Use another method to get a hold of parent nodes

		path = self.REQUEST['PARENTS']
		html_path = []

		for element_index in range(len(path)):

			element = path[element_index]

			if hasattr(element.aq_base, 'stop_breadcrumbs') and \
				element.stop_breadcrumbs:

				# If we're told that this is as far as it goes..

				break

			if hasattr(element.aq_base, 'name_title_or_id') and \
				callable(element.name_title_or_id):

				title = element.name_title_or_id()

			else:

				title = element.title_or_id()

			title = html_quote(title)

			html_path.append("""<a href="%s">%s</a>""" % \
						(element.absolute_url(), title))

		html_path.reverse()

		return """<span class="navigationbaritem">%s</span>""" % \
				string.join(html_path, separator_html)

	navigation_bar = navigation


	def footer(self, context):
		""

		return ""


	def product_installed(self, product_name):
		"""A method that returns a truth value if given
		product is installed."""

		if product_name in self.Control_Panel.Products.objectIds():

			return 1

		else:

			return 0


	def sneak_peak(self):
		"""Brings up a useful display of the object."""

		# Note:
		#
		# This makes all the properties defined available to
		# all users with access to the object.

		# Thanks to Steve Alexander for the aq_base.__class__.__name__
		# tip.

		return_string = "<class %s at %s" % \
					(self.aq_base.__class__.__name__,
						self.relative_url())

		for property in self._properties:

			property_id = property['id']
			value = eval("self.aq_base.%s" % property_id)

			return_string = return_string + "\n\t%s (%s): %s" % \
				(property_id, type(value), value)

		else:

			return_string = return_string + '\n>'

		return return_string


	def render_date(self, *args):
		"""Returns an instance specific format of the time."""

		# Hack to circumvent ZCatalog exception, FIXME

		if len(args) == 0:

			return None

		else:

			datetime = args[0]

		try:

			return datetime().strftime('%Y-%m-%d %H:%M')

		except AttributeError:

			return datetime.strftime('%Y-%m-%d %H:%M')


	make_date = render_date


	def relative_url(self):
		"""Returns the URL relative to the hostname and port."""

		return string.join(self.getPhysicalPath(), '/')


	def resolve_relative_url(self, relative_url):
		"""Method that resolves a URL assuming it is
		in the local namespace."""

		return self.restrictedTraverse(path=relative_url)


	def get_parent_node_title(self):
		"""Returns the title of the parent node."""

		return self.getParentNode().title


	def get_parent_node_relative_url(self):
		"""Returns the relative url of the parent node."""

		return self.getParentNode().relative_url()


	def get_parent_node_id(self):
		"""Returns the id of the parent node, used for
		ZCatalog indexing."""

		return self.getParentNode().id


	def local_frame_navigation(self):
		"""Wrapper for local_navigation."""

		return self.local_navigation()


	def get_nearest_object_by_meta_type(self, meta_type):
		"""Returns the nearest instance with given meta type."""

		instance = None

		reference = self

		while 1:

			if reference == None:

				# If we've come to the end of the line

				break

			try:

				instances = \
					reference.objectValues(meta_type)

			except AttributeError:

				# We've come to the Application instance

				break

			if instances:

				# If we found objects on the way

				instance = \
					instances[0]

				break

			reference = reference.getParentNode()

		return instance


	def get_all_objects_by_meta_type(self, meta_type):
		"""Returns all objects in acquisition path of given meta type."""

		return map(lambda x: x[-1],
			self.ZopeFind(self,
				obj_metatypes=[meta_type],
				search_sub=1))


	def export_as_understandable_xml(self):
		"""Exports the instance in a 'pure' xml format."""

		raise NotImplemented
