###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.3 $'


import os
import popen2
import thread
import string
import tempfile
import time

debug = 1

class fetchmail:
	"""An interface to fetchmail, which allows us to
	instruct fetchmail, as well as get an indication of
	the status of the last fetchmail process (if we're not
	currently running a process)."""

	# This may be '/usr/bin/fetchmail' etc.

	fetchmail_path = 'fetchmail'

	# Semantically, this is kinda sweet.  However, when we're supposed
	# to render the status message, we may have to iterate over three
	# different dictionaries to find the right message..

	misc_codes = {
		-1:'Fetchmail has not yet been used',
		-2:'Mail is currently being fetched'
	}

	success_codes = {
		0:'One or more messages were successfully retrieved',
		1:'No new messages, no messages retrieved'
	}

	fatal_error_codes = {
		2:'Socket error',
		3:'User authentication error',
		4:'Fatal protocol error detected',
		5:'Syntax error in configuration file',
		6:'Run control file had bad permissions',
		7:'Fetchmail timed out, or server error',
		8:'Client side exclusion error, another fetchmail'\
			' may be running',
		9:'Server responded lock busy, try again later',
		10:'Fetchmail failed on SMTP port opening, or transaction',
		11:'Fatal DNS error, error encountered upon lookup',
		12:'BSMTP batch file could not be opened',
		13:'Poll terminated by fetch limit',
		23:'Internal error, see standard error stream for info',
		127:'Fetchmail program not found, install it'
	}


	def __init__(self, server, username, local_username,
				 options=[], spam_error_codes=[],
				 protocol='auto', uidl=''):
		"""This initializes the fetchmail class."""

		self.server = server
		self.username = username
		self.local_username = local_username
		self.options = options
		self.protocol = protocol
		self.spam_error_codes = spam_error_codes
		self.uidl = uidl
		self.status = -1
		self.log = []


	def success(self):
		"""Returns a boolean indicating whether the last mail
		retrieval attempt succeeded."""

		# On second thought, is it right to assume that the misc_codes
		# are successful?

		if self.status < 2:
			return 1
		else:
			return 0


	def status_message(self):
		"""Returns a status message, giving us indication of the last
		mail retrieval attempt."""

		for error_dictionary in (self.misc_codes, self.success_codes,
			self.fatal_error_codes):

			if error_dictionary.has_key(self.status):
				return error_dictionary[self.status]

		else:

			raise 'UnknownFetchmailExitCode', self.status


	def version(self):
		"""Returns the fetchmail version."""

		return os.popen('fetchmail --version').readline()


	def debug(self):
		"""Returns information for debugging."""

		fetchmail_pipe = popen2.Popen3('fetchmail --version -f -')
		fetchmail_pipe.tochild.write(self.make_config('password'))
		fetchmail_pipe.tochild.close()

		return fetchmail_pipe.fromchild.read()


	def kill(self):
		"""Tries to kill the currently running fetchmail process."""

		# Should we try to report back some sort of status as well?
		# Is there a need for it?

		os.system('fetchmail -q')


	def make_config(self, password):
		"""Builds the configuration data."""

		configuration = """set properties ""
		poll  %s with proto %s
		user %s there with password %s is %s here
		""" % (self.server, self.protocol,
			   self.username, password, self.local_username)

		# FIXME, we're satisfied with SSL for now

		#if self.options:
		#
		#	configuration = configuration + "options %s" % \
		#	string.join(self.options,' ')

		if self.ssl:
			configuration = configuration + "options ssl"

			if self.keep:
				configuration = configuration + ' keep'
		else:

			if self.keep:
				configuration = configuration + 'options keep'

		return configuration


	def fetch(self, password, run_as_thread=0):
		"""Wrapper for mail retrieval."""

		if self.status == -2:
			raise 'Error', self.misc_codes[self.status]

		if run_as_thread:
			thread.start_new_thread(self._fetch,(password,))

		else:
			self._fetch(password)


	def _fetch(self, password):
		"""The method that does the actual retrieval of messages."""

		self.status = -2
		fetchmail_command = self.fetchmail_path + ' -f -'

		# If we're fetching from a server with a protocol specified
		# as 'auto', how will we then be able to store uids?

		if self.protocol == 'pop3' and self.uidl:

			uid_file = open(tempfile.mktemp(),'w')
			uid_file.write(self.uidl)
			uid_file.close()
			
			fetchmail_command = fetchmail_command + \
				(' -i %s' % uid_file.name)

		fetchmail_pipe = popen2.Popen3(fetchmail_command,
					capturestderr=1)

		if debug: print 'opened pipe..'
		stdin = fetchmail_pipe.tochild
		if debug: print 'trying to write to pipe..'
		stdin.write(self.make_config(password))
		if debug: print 'trying to flush pipe..'
		stdin.flush()
		if debug: print 'trying to close pipe..'
		stdin.close()
		if debug: 'waiting for fetchmail to finish..'

		lines = []
		lines2 = []

		for line in fetchmail_pipe.fromchild.readlines():
			print line
			lines.append(line)

		for line in fetchmail_pipe.childerr.readlines():
			#print line
			lines2.append(line)

		poll_result = fetchmail_pipe.wait()

		# Why does popen2.Popen3 return 4 times the original exit
		# code anyway?

		if poll_result > 255:

			poll_result = poll_result / 256

		if poll_result in self.success_codes.keys():

			if debug: print poll_result
			if self.uidl:

				uid_file = open(uid_file.name,'r')
				uid_file.seek(len(self.uid))
				self.uid = self.uid + uid_file.read()

				uid_file.close()
				os.unlink(uid_file.name)

		else:

			# Should we handle timeouts etc. here?
			# For example, sleep for a certain amount of time
			# and then try again?

			# FIXME

			if poll_result == 8:

				if debug: print 'sleeping.. %s' % poll_result

				time.sleep(60)
				self._fetch(password)

			if poll_result == 5:

				print self.make_config('password')

			print 'Error', 'fetchmail: %s' % poll_result

		self.log.append(poll_result, lines, lines2)

		self.status = poll_result


if __name__ == '__main__':

	fetchmail_instance = fetchmail('esol.no','morten',
				'morten',['flush','ssl','fetchall'])
	fetchmail_instance.ssl = 1
	fetchmail_instance.keep = 1