###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.4 $'


import gateway
import smtplib
from Products.RFC822Message import rfc822_message_parse
import fetchmail

from Globals import DTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
from Products.WarpFramework import base_item

manage_add_mail_gateway_form = \
	DTMLFile('dtml/mail_gateway_add', globals())
add_mail_gateway_redirect = \
	DTMLFile('dtml/mail_gateway_add_redirect', globals())


def manage_add_mail_gateway(self, server, protocol, port,
			smtp_server, smtp_port, username,
			local_username, REQUEST=None, RESPONSE=None):
	"""Add an instance of the mail_gateway class."""

	try:
		port = int(port)
	except ValueError:
		port = None

	try:
		smtp_port = int(smtp_port)
	except ValueError:
		smtp_port = None


	new_id = str(self.get_unique_id())

	self._setObject(new_id, mail_gateway(
					new_id,
					server, protocol, port, username,
					smtp_server,smtp_port, local_username
					))

	if REQUEST is not None:
		RESPONSE.redirect(self.absolute_url())
				

def parse_port(port):
	"""Method that abstracts port parsing for us,
	 returns None if a sane number can't be found."""

	try:
		parsed_port = int(port)
		if not parsed_port > 0:
			raise ValueError, 'Port number not greater than 0'
		return parsed_port
	except ValueError:
		return None
	except TypeError:
		return None


class mail_gateway(gateway.gateway,
		Products.ZCatalog.CatalogAwareness.CatalogAware,
		OFS.Folder.Folder,
		Persistent,
		Acquisition.Implicit,
		AccessControl.Role.RoleManager,
		base_item.base_item,
		fetchmail.fetchmail
		):
	"""Manages mail accounts, and settings for retrieval of messages."""

	meta_type = 'MailGateway'
	icon = 'misc_/GUM/Folder_icon.gif'

	_properties = (
		{'id': 'server', 'type':'string'},
		{'id': 'protocol', 'type':'string'},
		{'id': 'port', 'type':'string'},
		{'id': 'local_username', 'type':'string'},
		{'id': 'smtp_server', 'type':'string'},
		{'id': 'smtp_port', 'type':'string'},
		{'id': 'ssl', 'type':'boolean'},
		{'id': 'keep', 'type':'boolean'},
		# FIXME
		#{'id': 'spam_error_codes', 'type':'tokens'}
	)

	meta_types = ()
	protocols = ('imap4','pop3','auto')


	option_descriptions = {
		'bounce':'Bounce \'illegal\' mails.',
		'ssl':'Enable SSL login/transfer.',
		'keep':'Don\'t delete downloaded messages.'
	}

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View MailGateway',
		 ('', 'index_html')),
		)

	index_html = DTMLFile('dtml/mail_gateway_index', globals())
	base_edit = mail_gateway_edit = \
		DTMLFile('dtml/mail_gateway_edit', globals())
	get_new_messages = \
		DTMLFile('dtml/mail_gateway_get_new_messages', globals())

	def __init__(self, id,
			server, protocol, port, username,
			smtp_server, smtp_port,
			local_username, spam_error_codes=[]):
		"""Initializes the class."""

		# HACK - FIXME

		self.status = -1
		self.uidl = ''
		self.id = id
		self.server = server
		#self.options = options
		self.spam_error_codes = spam_error_codes

		if protocol not in self.protocols:
			pass
			#raise 'ValueError', \
			#	'Protocol "%s" not supported' % protocol
        
		self.protocol = protocol

		try:
			self.port = int(port)
		except ValueError:
			self.port = None
		except TypeError:
			self.port = None

		# If the SMTP server isn't specified, we assume that it is
		# located at the same host as the mail account.

		if smtp_server:
			self.smtp_server = smtp_server
		else:
			self.smtp_server = self.server

		try:
			self.smtp_port = int(smtp_port)
		except ValueError:
			self.smtp_port = None

		except TypeError:
			self.smtp_port = None

		self.username = username
		self.local_username = local_username
		self.ssl = 0
		self.keep = 1
		self.log = []

		try:
			self.index_object()
		except AttributeError:
			# No ZCatalog instance available.
			pass


	def get_new_messages_post(self, password, REQUEST=None, RESPONSE=None):
		"""Here we start up fetchmail for message retrieval."""

		self.fetch(password, run_as_thread=0)

		if RESPONSE and REQUEST:
			return self.status_message()
			#RESPONSE.redirect(self.get_gum().absolute_url())
