###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.7 $

from Globals import DTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
from Products.WarpFramework import base_item
import message_filter
import string


message_container_add_redirect = DTMLFile(
			'dtml/message_container_add_redirect', globals())
manage_add_message_container_form = DTMLFile(
			'dtml/message_container_add', globals())


def manage_add_message_container(self, title, description, type=[0],
				id=None, REQUEST=None, RESPONSE=None):
	"""Add an instance of the message_container class"""

	message_container_id = self.id

	for type in type:
		new_id = id or self.get_unique_id()
		self._setObject(new_id, message_container(new_id,
				title, description, int(type),
				message_container_id=message_container_id))

	if RESPONSE is not None:
		RESPONSE.redirect(self.absolute_url())
	else:
		return getattr(self, new_id)


class message_container(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	Products.BTreeFolder.BTreeFolder.BTreeFolder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Container we use to group messages."""

	meta_type = 'MessageContainer'
	icon = 'misc_/GUM/Folder_icon.gif'
	default_catalog = 'Catalog'
	help_name='message_container.stx'

	use_local_navigation = 0
	deleteable_object = 1
	editable_object = 1

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'type', 'type':'int', 'select_widget':'types'}
	)
		

	types = {
			0:'Incoming',
			1:'Outgoing',
			2:'Draft',
			3:'Template'
	} 

	_properties = (
		{'id':'title', 'type':'string'},
	)

	meta_types = (
		{'name':'MessageFilter',
			'action':'manage_add_message_filter_form'},
		{'name':'MessageContainer',
			'action':'manage_add_message_container_form'},
	)

	manage_add_message_container_form = manage_add_message_container_form
	manage_add_message_container = manage_add_message_container

	manage_add_message_filter_form = \
		message_filter.manage_add_message_filter_form
	manage_add_message_filter = message_filter.manage_add_message_filter
	message_filter_add_redirect = \
		message_filter.add_message_filter_redirect

	__ac_permissions__ = (
		('View management screens',
			('manage_tabs', 'manage_main')
		),
		('Change Permissions',
			('manage_access',)
		),
		('View MessageContainers',
			('',)
		),
	)

	index_html = DTMLFile('dtml/message_container_index', globals())
	message_filters = DTMLFile('dtml/message_container_message_filters',
					globals())


	def __init__(self, id, title='', description='', type=0,
			message_container_id=None):
		"Initialize the class instance"

		Products.BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)

		self.id = id
		self.title = title
		self.description = description
		self.type = type

		# So we can find out who's the parent in ZCatalog
		self.message_container_id = message_container_id
		self.number_of_unread_messages = 0
		self.number_of_messages = 0


	def get_message_by_path(self, path):
		"""Returns the message specified in the path."""

		return self.restrictedTraverse(path)


	def get_message_container(self):
		"""Returns self."""

		return self


	def get_message_containers(self):
		"""Returns all message containers contained."""

		message_containers = []

		for catalog_brain in self.Catalog.searchResults(
			meta_type='MessageContainer',
			message_container_id=self.id):

			message_containers.append(self.unrestrictedTraverse(
				self.Catalog.getpath(catalog_brain.getRID())))

		return message_containers


	def get_nested_title(self):
		"""Returns a title based on what the parent nodes are."""

		parent_message_containers = []
		reference = self

		while 1:

			reference = reference.getParentNode()

			if reference.meta_type == 'MessageContainer':
				parent_message_containers.\
					append(reference.title)
			else:
				break

		if parent_message_containers:
			parent_message_containers.reverse()
			return string.join(parent_message_containers,
					'/') + '/' + self.title
		else:
			return self.title


	def get_message_ids(self):
		"""Returns a list of ids of the messages contained."""

		object_ids = self.objectIds('RFC822Message')
		object_ids.sort()
		object_ids.reverse()

		return object_ids


	def is_draft_folder(self):
		"""Returns a truth value if self is a draft type
		message container."""

		if self.type == 2:
			return 1
		else:
			return 0

	def is_template_folder(self):
		"""Returns a truth value if self is a template type
		message container."""

		if self.type == 3:
			return 1
		else:
			return 0


	def get_message_filters(self):
		"""Returns a list of message filters, sorted by priority."""

		message_filters = []
		catalog = self.Catalog

		for catalog_brain in catalog.searchResults(
			meta_type='MessageFilter',
			message_container_id=self.id
			):

			message_filters.append(self.unrestrictedTraverse(
				catalog.getpath(catalog_brain.getRID())))

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters


	def get_number_of_mail_messages(self):
		"""Returns the number of messages this object contains."""

		number_of_messages = \
			len(self.Catalog.searchResults(
				meta_type='RFC822Message',
				message_container_id=self.id))

		for container in self.get_message_containers():

			number_of_messages = number_of_messages + \
				container.get_number_of_mail_messages()

		return number_of_messages


	def get_number_of_unread_mail_messages(self):
		"""Returns the number of unread messages."""

		number_of_unread_messages = \
			len(self.Catalog.searchResults(
				meta_type='RFC822Message',
				message_container_id=self.id,
				read=0))

		for container in self.get_message_containers():

			number_of_unread_messages = \
				number_of_unread_messages + \
				container.get_number_of_unread_mail_messages()

		return number_of_unread_messages


	def get_messages(self):
		"""Returns a list of messages contained."""

		return self.objectValues('RFC822Message')


	def manage_delete(self, ids=[], REQUEST=None, RESPONSE=None):
		"""Moves the messages to a trash folder unless it is the
		trashfolder itself, in which case the messages are deleted."""

		trash_folder = self.get_trash_folder()

		if self.id == trash_folder.id:
			for id in ids:
				self.manage_delObjects(ids=[id])
		else:
			for id in ids:
				if self[id].meta_type == 'RFC822Message':
					trash_folder.paste(self[id].cut())
				else:
					self.manage_delObject(ids=[id])

		if RESPONSE is not None:
			RESPONSE.redirect(self.absolute_url())


default__class_init__(message_container)

