###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.5 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import DTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.SimpleItem
import Products
from Products.WarpFramework import base_item
import re
import string

add_message_filter_redirect = \
	DTMLFile('dtml/message_filter_add_redirect', globals())
manage_add_message_filter_form = \
	DTMLFile('dtml/message_filter_add', globals())


def manage_add_message_filter(self, title, header_name='', regexp_rule='',
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the message_filter class."""

	new_id = str(self.get_unique_id())
	priority = self.get_new_priority()
	message_container_id = self.id

	self._setObject(new_id,
			message_filter(new_id, title, header_name,
				regexp_rule, priority,
				message_container_id=message_container_id))

	if RESPONSE is not None:
	        RESPONSE.redirect(self.absolute_url() + '/message_filters')


class message_filter(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	Persistent,
	Acquisition.Implicit,
	OFS.SimpleItem.SimpleItem,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Filter we use to determine which folder the message goes to."""

	meta_type = 'MessageFilter'
	icon = 'misc_/GUM/dtmldoc.gif'
	default_catalog = 'Catalog'
	help_name = 'message_filter.stx'

	header_names = ('from', 'to', 'cc', 'x-priority',
				'any', 'any_address_field')

	editable_object = 1

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'header_name', 'type':'string'},
		{'id':'regexp_rule', 'type':'string'},
		{'id':'match_any_header', 'type':'boolean'},
		{'id':'match_any_address_field', 'type':'boolean'},
		{'id':'priority', 'type':'int'}
	)


	__ac_permissions__ = (
		('View management screens',
			('manage_tabs', 'manage_main',)
		),
		('Change Permissions',
			('manage_access',)
		),
		('View MessageFilter',
			('',)
		),
	)

	index_html = DTMLFile('dtml/message_filter_index', globals())
	base_edit = DTMLFile('dtml/message_filter_edit', globals())

	def base_edit_properties(self, title, header_name,
			regexp_rule, RESPONSE):
		"""Updates self.."""

		self.title = title
		self.header_name = header_name
		self.regexp_rule = regexp_rule

		if header_name == 'any':
			self.match_any_header = 1
			self.match_any_address_field = 0
		if header_name == 'any_address_field':
			self.match_any_address_field = 1
			self.match_any_header = 0

		self.link()
		RESPONSE.redirect(self.absolute_url())


	def render_priority_widget(self):
		"""Renders an HTML widget for selecting message
		filter priority."""

		select_widget = """<select name="priority:int">"""

		for message_filter in self.get_message_filters():
			select_widget = select_widget + \
				"""<option value="%s">%s</option>""" % \
					(message_filter.priority,
					message_filter.title_or_id())

		return select_widget + "</select>"


	def __init__(self, id, title, header_name, regexp_rule, priority,
			message_container_id=None):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.header_name = header_name

		if header_name == 'any':
			self.match_any_header = 1
		else:
			self.match_any_header = 0

		if header_name == 'any_address_field':
			self.match_any_address_field = 1
		else:
			self.match_any_address_field = 0

		self.regexp_rule = regexp_rule
		self.priority = priority
		self.message_container_id = message_container_id

		self.link()


	def link(self):
		"""Compiles the regexp_rule."""

		self._regexp_rule = re.compile(re.escape(
					string.strip(self.regexp_rule)))


	def match(self, headers):
		"""Returns a truth value if header contains what we're
		looking for."""

		if self.match_any_header:
			for header_value in headers.values():
				match_object = self._regexp_rule.search(
						header_value)
				if match_object:
					return match_object
		elif self.match_any_address_field:
			for key in headers.keys():

				if string.lower(key) in ('to','cc','from'):
					header_value = headers[key]
					match_object = \
						self._regexp_rule.search(
							header_value)
				else:
					continue

				if match_object:
					return match_object

			else:
				return match_object


		else:
			match_object = self._regexp_rule.search(
						headers[self.header_name])

		return match_object


	def get_message_container(self):
		"""Returns the message container we're referencing."""

		return self.getParentNode()


default__class_init__(message_filter)
