###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.4 $

from Globals import DTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import news_message_indicator
from Products.WarpFramework import base_item


news_group_folder_add_redirect = \
	DTMLFile('dtml/news_group_folder_add_redirect', globals())
manage_add_news_group_folder_form = \
	DTMLFile('dtml/news_group_folder_add', globals())


def manage_add_news_group_folder(self, id, title, REQUEST=None, RESPONSE=None):
	"""Add an instance of the NewsGroupFolder class."""

	self._setObject(id, news_group_folder(id, title))

	if RESPONSE is not None:
		RESPONSE.redirect(self.absolute_url())


class news_group_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Folder we use to store newsgroup messages in."""

	meta_type = 'NewsGroupFolder'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
	)

	meta_types = (
		{'name':'NewsMessageIndicator',
			'action':'manage_add_news_message_indicator_form'
		},
	)

	manage_add_news_message_indicator_form = \
		news_message_indicator.manage_add_news_message_indicator_form
	manage_add_news_message_indicator = \
		news_message_indicator.manage_add_news_message_indicator
	news_message_indicator_add_redirect = \
		news_message_indicator.news_message_indicator_add_redirect

	__ac_permissions__ = (
		('View management screens',
			('manage_tabs', 'manage_main')
		),
		('Change Permissions',
			('manage_access',)
		),
		('Add NewsMesssageIndicator',
			('manage_add_news_message_indicator',
			 'manage_add_news_message_indicator_form')
		),
		('View NewsGroupFolders',
			('',)
		),
	)

	index_html = DTMLFile('dtml/news_group_folder_index', globals())

	def __init__(self, id, title=''):
		"Initialize the class instance"

		self.id = id
		self.last_marked_message = '0'

		try:
			self.index_object()
		except AttributeError:
			# No ZCatalog instance available..
			pass


	def retrieve(self, RESPONSE=None):
		"""Marks new messages for this group from nearest
		NewsHost instance."""

		news_host = self.get_news_host()
		news_group = news_host[self.id]

		message_ids = news_group.objectIds('RFC822Message')

		try:
			start_index = message_ids.index(
					self.last_marked_message)
			message_ids = message_ids[start_index + 1:]
		except ValueError, value:
			#if int(self.last_marked_message) > \
			#	int(message_ids[-1]):
			pass

		for message_id in message_ids:
			news_message_indicator.\
				manage_add_news_message_indicator(
				self, message_id, 0)
		else:
			try:
				self.last_marked_message = message_id
			except NameError:
				pass

		if RESPONSE is not None:
			RESPONSE.redirect(self.absolute_url())


	def manage_tick(self, ids, REQUEST=None, RESPONSE=None):
		"""Method that ticks (marks them as .. 'read later'
		or 'note' etc.)."""

		for message_id in ids:
			self[message_id].status = 1

		RESPONSE.redirect(self.absolute_url())


	def get_news_message_indicators(self):
		"""Returns a list of message indicators
		contained in this folder."""

		return self.objectValues('NewsMessageIndicator')


	def get_number_of_messages(self):
		"""Returns the number of 'messages' this folder contains."""

		return len(self.objectIds())


default__class_init__(news_group_folder)
