###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.2 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

from OFS import ObjectManager

class object_manager(ObjectManager.ObjectManager):
	"""Used for grouping object management related methods and attributes."""

	object_sheet = ()
	object_sheets = ()

	def get_object_listing(self, meta_type, status=0):
		"""Returns a set of objects."""

		objects = []
		object_sheet = self.get_object_sheet(meta_type)

		for object in self.get_objects_by_meta_type(meta_type, status=status):

			object_data = []

			for entry in object_sheet:

				object_data.append(eval('object.%s' % entry[1]))

			objects.append({
				'id':object.id,
				'absolute_url':object.absolute_url(),
				'data':object_data}
			)

		return objects


	def get_number_of_objects(self, meta_type, status=0, **kw):
		"""Returns the number of objects with the given meta_type."""

		return len(self.get_objects_by_meta_type(meta_type, status=status))


	def get_objects_by_meta_type(self, meta_type, status=0):
		"""Returns a listing of objects."""

		if hasattr(self.aq_base, '_virtual_folder_requirements'):

			if hasattr(self.aq_base, 'catalog_expression'):

				catalog = eval('self.%s' % self.catalog_expression)

			else:

				catalog = self.Catalog

			requirements = {'meta_type':meta_type}

			for requirement in self._virtual_folder_requirements:

				key = requirement[0]
				value = eval(requirement[1])

				requirements[key] = value

			brains = catalog.searchResults(requirements)
			return map(catalog.getobject, map(lambda x: x.getRID(), brains))

		else:

			# Implement

			return self.objectValues(meta_type)


	def manage_delete(self, ids=[], REQUEST=None, RESPONSE=None):
		"""Deletes given ids."""

		if type(ids) is type(''):

			ids = [ids]

		self.manage_delObjects(ids)

		if REQUEST is not None:

			if REQUEST.has_key('__referrer'):

				RESPONSE.redirect(REQUEST['__referrer'])

			else:


				RESPONSE.redirect(self.absolute_url())


	def manage_delete_all(self, REQUEST=None, RESPONSE=None):
		"""Method that deletes all messages in folder."""

		if REQUEST is not None:

			if REQUEST.has_key('confirm') and \
				REQUEST['confirm']:

				pass

			else:

				RESPONSE.redirect(self.absolute_url())
				return

		self.manage_delete(ids=self.objectIds())

		RESPONSE.redirect(self.absolute_url())