###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.1.1.1 $'


import imaplib
import string
import mail_wrapper

class imap4_wrapper(mail_wrapper.mail_wrapper):
    """A protocol wrapper."""

    def __init__(self,server,port=None):

        mail_wrapper.mail_wrapper.__init__(self,server,port)
        
        if self.port:

            self.connection = imaplib.IMAP4(self.server,self.port)

        else:

            self.connection = imaplib.IMAP4(self.server)


    def login(self,username,password):
        """Logins to IMAP server."""

        self.connection.login(username,password)
        self.connection.select()

    def list(self):
        """Returns a list of unique message ids."""

        result = self.connection.uid('SEARCH','ALL')
        
        return map(int,string.split(result[-1][-1]))
            

    def delete(self,unique_message_id):
        """Marks message as deleted."""

        self.connection.uid('STORE',str(unique_message_id),'+FLAGS',
                            '\\DELETED')


    def retrieve(self,unique_message_id):
        """Retrieves specified message.

        Returns the message in a list, one entry per line."""

        return self.connection.uid('FETCH',str(unique_message_id),
                                   '(RFC822)')[-1]


    def head(self,unique_message_id):
        """Retrieves message header."""

        return self.connection.uid('FETCH',str(unique_message_id),'HEAD')


    def quit(self,expunge=1):
        """Quits the connection.  If the expunge parameter is true,
        all messages marked as deleted are deleted."""

        if expunge:

            self.connection.expunge()

        self.connection.logout()
