###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


# A regular expression that will find filename="something",
# name="something" values.  It may be quoted in either
# single (') or double (") quotes.

filename_quoted_regexp = re.compile(r'(filename=|name=)' \
                                    r'((\'.*?\')|' \
                                    r'(\".*?\"))'
                                    ,re.I)

filename_unquoted_regexp = re.compile('(filename=|name=)' \
									  r'(\w|;)*',re.I) # FIXME

class mime_part_old:
    """This class handles message parts as they are delivered
    by the multifile.multifile class."""


    def __init__(self,lines):
		
        self.lines = lines
        self.headers = {}
        self.data = 'No body.'
        self.content_type = ''
        self.main_type = ''
        self.sub_type = ''
        self.filename = 'default_filename'

        # We split up the part in headers and data (body).

        separator = self.lines.index('\012')

        # We handle and store all the headers.
        
        for line in self.lines[:separator]:

            line = mimify.mime_decode_header(line)

            split = string.find(line,':')

            name = line[:split]
            content = line[split:][1:-1]

            self.headers[string.lower(name)] = content[1:]

        # We try to find the content-type of the mime part,
        # if we can't, what content-type should we assume? UPDATE

        if self.headers.has_key('content-type'):

            comma = string.find(self.headers['content-type'],';')

            if comma > -1:

                self.content_type = string.strip(string.lower(
                    self.headers['content-type'])[:comma])

            else:

                self.content_type = string.strip(string.lower(
                    self.header['content-type']))

            self.main_type = self.content_type[:string.find(
                self.content_type,'/')]
            self.sub_type = self.content_type[string.find(
                self.content_type,'/')+1:]

        # We try to find the filename of the mime part.
        # The content-disposition header superseedes the
        # content-type header.

        temporary_filename_storage = []

        for header_name in self.headers.keys():

            match_object = filename_quoted_regexp.search(
                self.headers[header_name])

            if not match_object:
                
                match_object = filename_unquoted_regexp.search(
                    self.headers[header_name])

            if match_object:

                temporary_filename_storage.append((header_name,
                                                   match_object.group()
                                                   ))

        if temporary_filename_storage:

            for filename in temporary_filename_storage:

                if filename[0] == 'content-disposition':

                    # We've found the best candidate, break.

                    break

                elif filename[0] == 'content-type':

                    # We've found an acceptable (?) candidate.

                    break

                else:

                    # We're satisifed with an unknown header.

                    break


            quote_match = quote_regexp.search(filename[1])


            if quote_match:

                self.filename = util.unquote_string(quote_match.group())

            else:

                # We've got a 'filename=filename.bin'.

                pass
                #misc.not_implemented()

        else:

                # We couldn't find a filename.

                pass

        # The remaing lines of the mime part are regarded
        # as the data (body).

        raw_data = string.join(self.lines[separator+1:],'')

        # We see if the data is encoded, if so, decode the
        # message.

        if self.headers.has_key('content-transfer-encoding'):

            encoding = string.lower(self.headers['content-transfer-encoding'])

            # If encoding is supported in the mimetools package.

            if encoding in mimetools.decodetab.keys():

                out, _in = cStringIO.StringIO(), cStringIO.StringIO(raw_data)

                mimetools.decode(_in,out,encoding)

                self.data = out.getvalue()

            # If the data is 'encoded' in it's original form.

            elif encoding in ('7bit','8bit'):

                self.data = raw_data

            else:

                raise 'UnknownEncodingException','Notify the application' \
                      ' maintainer'

        else:

            self.data = raw_data
