###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.4 $'


import poplib
import string
import mail_wrapper
import cStringIO


class pop3_wrapper(mail_wrapper.mail_wrapper):
	"""A protocol wrapper."""

	def __init__(self,server,port=None):

		mail_wrapper.mail_wrapper.__init__(self,server,port)

		if port:
			self.connection = poplib.POP3(self.server,self.port)
		else:
			self.connection = poplib.POP3(self.server)

		self.retrieved_messages = []
		self.deleted_messages = []
		self.session_retrieved = []

	def login(self, username, password):
		"""Logins to POP server."""

		self.connection.user(username)
		self.connection.pass_(password)

	def list(self):
		"""Returns a list of message ids."""

		return map(lambda x: string.split(x)[0], 
				self.connection.list()[1])

	def retrieve_new_ids(self):
		"""Retrieves all new message ids."""

		result = self.list()[1]
		# The combination between message id and the
		# number of octets gives us a reasonably good
		# chance of retrieving new messages if someone
		# has accessed the mailbox and deleted any messages.
		#
		# We can't base our message handling on uidl
		# because not all POP3 servers support it.
		# (They are optional commands).
		# FIXME

		if self.retrieved_messages == result[:len(
				self.retrieved_messages)]:
			return result[:len(self.retrieved_messages)]
		else:
			self.retrieved_messages = []
			self.deleted_message = []

		return result


	def retrieve_all_ids(self):
		"""Retrieves all message ids."""

		return self.list()
    

	def delete(self, message_id):
		"""Marks specified message as deleted."""

		real_id = string.split(message_id)[0]
		self.connection.dele(real_id)
        

	def delete_all(self):
		"""Deletes all messages in the mailbox."""

		for id in self.list():
			self.delete(id)
		self.quit()


	def retrieve(self, message_id):
		"""Retrieves specified message."""

		real_message_id = string.split(message_id)[0]
		message = self.connection.retr(real_message_id)
		self.retrieved_messages.append(message_id)
		message_fp = cStringIO.StringIO(string.join(message[1],'\n'))

		return mail_wrapper.dummy_mailbox_message(message_fp)


	def quit(self, expunge=1):
		"""Quits the connection.  If the expunge parameter is true,
		all messages marked as deleted are deleted."""

		if not expunge:
			self.connection.rset()

		self.connection.quit()


class pop3_session(pop3_wrapper):

	def __init__(self, server, port=None, delete_retrieved_messages=1):

		pop3_wrapper.__init__(self, server, port)
		self.delete_retrieved_messages = delete_retrieved_messages
		self.retrieved_messages = []


	def login(self, username, password):
		"""Logs in the user and updates the message id list."""

		# We have to override the login method here so that
		# we can update the id list.

		pop3_wrapper.login(self, username, password)
		self.id_list = self.list()


	def next(self):
		"""Returns the next message in the list."""

		try:
			id = self.id_list.pop()
		except IndexError:
			return None

		message =  self.retrieve(id)

		return message


	def quit(self):
		"""Deletes the messages if told to do so,
		and quits the connection."""

		if self.delete_retrieved_messages:
			for id in self.retrieved_messages:
				self.delete(id)

		pop3_wrapper.quit(self)


if __name__ == '__main__':

	session = pop3_session('localhost')
	session.login('test123','test123')
