###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.3 $

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
#from Products.BTreeFolder.BTreeFolder import BTreeFolder
from Products.WarpFramework import base_item
import message_filter


manage_add_virtual_message_container_form = HTMLFile('dtml/virtual_message_container_add', globals())


def manage_add_virtual_message_container(self, title, description, type=[0], id=None, REQUEST=None, RESPONSE=None):
	"""Add an instance of the virtual_message_container class."""

	for type in type:

		new_id = id or self.get_unique_id()

		self._setObject(new_id, virtual_message_container(new_id, title, description, int(type)))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class virtual_message_container(
	#Products.BTreeFolder.BTreeFolder.BTreeFolder,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Container we use to access messages stored elsewhere in the system, based on rules."""

	meta_type = 'VirtualMessageContainer'
	icon = 'misc_/GUM/Folder_icon.gif'
	default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'referenced_containers', 'type':'list'},
		{'id':'type', 'type':'int', 'select_widget':'types'}
		)
		

	types = {
			0:'Incoming',
			1:'Outgoing',
			2:'Draft',
			3:'Template'
		} 

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
			{'name':'MessageFilter', 'action':'manage_add_message_filter_form'},
			)

	manage_add_message_filter_form = message_filter.manage_add_message_filter_form
	manage_add_message_filter = message_filter.manage_add_message_filter

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		#('Add Message',
		# ('manage_add_message', 'manage_add_message_orm')),
		('View MessageContainers',
		 ('',)),
		)

	index_html = HTMLFile('dtml/message_container_index', globals())
	message_filters = HTMLFile('dtml/virtual_message_container_message_filters', globals())


	def __init__(self, id, title='', description='', type=0):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.description = description
		self.type = type

		# ZCatalog chokes because it uses absolute url.  Hrm,
		# why does it use absolute_url anyway?
		from Products.ZCatalog.ZCatalog import ZCatalog
		self._setObject('Catalog', ZCatalog(self, 'Catalog', '', self))


	def is_draft_folder(self):
		"""Returns a truth value if self is a draft type message container."""

		if self.type == 2:

			return 1

		else:

			return 0


	def is_template_folder(self):
		"""Returns a truth value if self is a template type message container."""

		if self.type == 3:

			return 1

		else:

			return 0


	def get_message_filters(self):
		"""Returns a list of message filters, sorted by priority."""

		message_filters = self.objectValues('MessageFilter')

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters


	def get_number_of_messages(self):
		"""Returns the number of messages this object contains."""

		return len(self.get_messages('RFC822Message'))


	def get_number_of_unread_messages(self):
		"""Returns the number of messages unread that this object contains."""

		return len(filter(lambda x: not x.read, self.get_messages()))


	def get_messages(self):
		"""Returns a list of messages contained."""

		# FIXME

		return self.objectValues('RFC822Message')


	def manage_delete(self, ids=[], RESPONSE=None):
		"""Moves the messages to a trash folder unless it is the trashfolder
		itself, in which case the messages are deleted."""

		# How should we handle this with virtuals?

		trash_folder = self.get_trash_folder()

		if self.id == trash_folder.id:

			self.manage_delObjects(ids=ids)

		else:

			for id in ids:

				trash_folder.paste(self[id].cut())

		if RESPONSE is not None:

			RESPONSE.redirect(self.absolute_url())

				
default__class_init__(virtual_message_container)

