###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.3 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_address
from Products.WarpFramework import base_item
import types


rfc822_address_container_add_redirect = HTMLFile('dtml/rfc822_address_container_add_redirect', globals())
manage_add_rfc822_address_container_form = HTMLFile('dtml/rfc822_address_container_add', globals())


def manage_add_rfc822_address_container(self, id, title, REQUEST=None):
	"""Add an instance of the rfc822_address_container class."""

	self._setObject(id, rfc822_address_container(id, title))

	if REQUEST is not None:

		return self.manage_main(self, REQUEST)


class rfc822_address_container(
	#Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Container used to group rfc822_address instances."""

	meta_type = 'RFC822AddressContainer'
	icon = 'misc_/RFC822Message/dtmldoc.gif'
	help_name = 'rfc822_address_container.stx'

	_properties = (
		{'id':'title', 'type':'string'},
	)

	"""
	meta_types = (
		{'name': 'RFC822Address', 'action':'manage_add_rfc822_address_form'},
		)
	"""

	manage_add_rfc822_address_form = rfc822_address.manage_add_rfc822_address_form
	manage_add_rfc822_address = rfc822_address.manage_add_rfc822_address
	rfc822_address_add_redirect = rfc822_address.rfc822_address_add_redirect

	r = address_book_interface = \
		HTMLFile('dtml/rfc822_address_container_address_book_interface', globals())
	address_book_interface_select_email = \
		HTMLFile('dtml/rfc822_address_container_address_book_interface_select_email', globals())

	manage_options = (
		{'label':'Edit', 'action':'manage_main'},
		{'label':'View', 'action':''},
	)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Attachment',
		 ('','render_data')),
		)

	index_html = HTMLFile('dtml/rfc822_address_container_index', globals())


	def __init__(self, id, title=''):
		"""Initialize the class instance."""

		self.id = id
		self.title = title


	def add_email(self, REQUEST, RESPONSE=None):
		"""Adds a set of email addresses."""

		try:

			emails = REQUEST['emails']

		except KeyError:

			emails = []

		if type(emails) is not types.ListType:

			emails = [emails]

		for email in emails:

			new_id = str(self.get_unique_id())

			self.manage_add_rfc822_address(new_id, email)

		if RESPONSE is not None:

			RESPONSE.redirect(self.get_rfc822_message().relative_url())


	def add_normal_email(self, REQUEST, RESPONSE):
		"""Adds all normal emails specified."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				self.manage_add_rfc822_address(str(self.get_unique_id()),
							self.get_address_book().get_contact(id).get_email().email)

		# Redirect to the mail message.

		RESPONSE.redirect(self.getParentNode().absolute_url())
		

	def add_address_and_update(self, REQUEST, RESPONSE):
		"""Updates the message and returns an add contact field."""

		self.getParentNode().update(REQUEST=REQUEST)

		RESPONSE.redirect(self.absolute_url() + '/address_book_interface')



	def add_group(self, group, RESPONSE):
		"""Adds the specified group."""

		contacts = self.get_address_book().get_contacts_from_group(group)

		for contact in contacts:

			new_id = self.get_unique_id()
			rfc822_address = contact.get_email()

			self.manage_add_rfc822_address(new_id, rfc822_address.email, '', '')

		RESPONSE.redirect(self.getParentNode().relative_url())


default__class_init__(rfc822_address_container)

