###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.3 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.SimpleItem
import OFS.content_types
import cStringIO
import mimetools
import rfc822_header_container
from Products.WarpFramework import base_item
import string
import util
from Products.WarpFramework import utils

rfc822_attachment_add_redirect = HTMLFile('dtml/rfc822_attachment_add_redirect',
					globals())
manage_add_rfc822_attachment_form = HTMLFile('dtml/rfc822_attachment_add',
						globals())

def manage_add_rfc822_attachment(self, file=None, data='', headers='',
					filename='', content_type='',
					RESPONSE=None):
	"""Add an instance of the rfc822_attachment class."""

	new_id = str(self.get_unique_id())

	# If we've got the RESPONSE object, this attachment was
	# submitted via the web.

	if RESPONSE is not None:

		data = file.read()
		filename = file.filename

		backslash_index = string.rfind(filename, '\\')

		# Now, this screws up unix filenames with backslahes
		# in them.. *ponder*

		if backslash_index > -1:

			filename = filename[backslash_index+1:]
			
		headers = {}
		content_type = OFS.content_types.guess_content_type(
				name=filename, body=data,
				default='application/octet-stream')[0]


	self._setObject(new_id,
		rfc822_attachment(new_id,
			data=data, content_type=content_type,
			filename=filename, headers=headers
		)
	)

	attachment_reference = self[new_id]

	# FIXME

	"""
	# So we don't get an AttributeError adding the headers container

	get_transaction().commit()

	attachment_reference.manage_add_rfc822_header_container('headers', '')

	for header_key in headers.keys:

		attachment_reference.headers.manage_add_rfc822_header(header_key, headers[header_key])
	"""



	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url() + '/')

	else:

		return attachment_reference


class rfc822_attachment(
	#Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Storage for message attachments."""

	meta_type = 'RFC822Attachment'
	icon = 'misc_/RFC822Message/Folder_icon.gif'

	meta_types = (
		{'name':'RFC822HeaderContainer',
		 'action':'manage_add_rfc822_header_container_form'},
		)

	manage_add_rfc822_header_container = rfc822_header_container.manage_add_rfc822_header_container

	_properties = (
		{'id':'data', 'type':'text'},
		{'id':'content_type', 'type':'string'},
		{'id':'filename', 'type':'string'},
		)

	manage_options = (
		{'label':'Edit', 'action':'manage_main'},
		{'label':'View', 'action':''},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Attachment',
		 ('','render_data')),
		)

	index_html = HTMLFile('dtml/rfc822_attachment_index', globals())
	manage_main = HTMLFile('dtml/rfc822_attachment_edit', globals())


	def __init__(self, id, data='', content_type='', filename='',
			headers={}):
		"""Initialize the class instance."""

		self.id = id
		self.data = data
		self.content_type = content_type
		self.filename = filename


	def encode(self):
		"""Returns encoded data."""

		out_file = cStringIO.StringIO()
		in_file = cStringIO.StringIO(self.data)
		in_file.seek(0)

		mimetools.encode(in_file, out_file, 'base64')

		return out_file.getvalue()


	def get_size(self):
		"""Returns the length of the data."""

		return len(self.data)


	def download(self, REQUEST=None, RESPONSE=None):
		"""Method for downloading / viewing attachment."""

		RESPONSE.setHeader('Content-Type', '%s; filename=%s' % \
					(self.content_type, self.filename))
		RESPONSE.setHeader('Content-Length', self.get_size())
		RESPONSE.setHeader('Content-Disposition', 'filename=%s' % self.filename)

		RESPONSE.write(self.data)


	def set_header(self, header_name, header_value):
		"""Sets an attachment header."""

		if hasattr(self.headers.aq_base, header_name):

			self.headers[header_name].value = header_value

		else:

			self.headers.manage_add_rfc822_header(header_name, header_value)


	def render_size(self, details=0):
		"""Returns a human-readable display of the size."""

		return utils.format_size(self.get_size(), details=details)


default__class_init__(rfc822_attachment)

