###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.3 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_attachment
import rfc822_address_container
import rfc822_address
import rfc822_header_container
from Products.WarpFramework import base_item, product_base_item
from Products.WarpFramework.utils import html_quote
import cStringIO
import mimetools
import mimify
import extended_mime_writer
import string
import DateTime
import util
from Products.WarpFramework import utils
from ZPublisher.HTTPRequest import html_quote

add_rfc822_message_redirect = HTMLFile('dtml/rfc822_message_add_redirect',
					globals())
manage_add_rfc822_message_form = HTMLFile('dtml/rfc822_message_add', globals())


def manage_add_rfc822_message(self, id, sender='', reply_to='', body='',
				subject='', message_id='', priority=1,
				readonly=None, REQUEST=None):
	"""Add an instance of the rfc822_message class."""

	self._setObject(id,
			rfc822_message(id, sender, body, subject,
				message_id=message_id, reply_to=reply_to,
				priority=priority, readonly=readonly))


class dummy_message_container:

	def __init__(self):

		self.id = ''
		self.title = "Don't copy to any folder"


	def get_nested_title(self):

		return self.title

	def absolute_url(self, *args, **kw):

		return ''

	def relative_url(self, *args, **kw):

		return ''


class rfc822_message(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	Products.BTreeFolder.BTreeFolder.BTreeFolder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""For storing structured RFC822 Messages."""

	meta_type = 'RFC822Message'
	product_name = 'RFC822Message'
	icon = 'misc_/RFC822Message/Folder_icon.gif'
	help_name = 'rfc822_message.stx'

	use_local_navigation = 0

	priorities = {
			0:'High',
			1:'Normal',
			2:'Low'
		}

	_properties = (
		{'id':'sender', 'type':'string'},
		{'id':'reply_to', 'type':'string'},
		{'id':'body', 'type':'text'},
		{'id':'readonly', 'type':'boolean'},
		{'id':'read', 'type':'boolean'},
		{'id':'message_container_id', 'type':'int'},
	)

	manage_add_rfc822_attachment_form = rfc822_attachment.manage_add_rfc822_attachment_form
	manage_add_rfc822_attachment = rfc822_attachment.manage_add_rfc822_attachment

	manage_add_rfc822_address_container_form = rfc822_address_container.manage_add_rfc822_address_container_form
	manage_add_rfc822_address_container = rfc822_address_container.manage_add_rfc822_address_container

	manage_add_rfc822_header_container = rfc822_header_container.manage_add_rfc822_header_container

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add RFC822Attachment',
		 ('manage_add_rfc822_attachment', 'manage_add_rfc822_attachment_form')),
		('Add RFC822Address',
		 ('manage_add_rfc822_address', 'manage_add_rfc822_address_form')),
		('View RFC822Messages',
		 ('',)),
		)

	index_html = HTMLFile('dtml/rfc822_message_index', globals())

	view_index_html = HTMLFile('dtml/rfc822_message_view_index', globals())
	edit_index_html = HTMLFile('dtml/rfc822_message_edit_index', globals())
	edit_attachments_html = HTMLFile('dtml/rfc822_message_edit_attachments', globals())


	def make_todo(self, REQUEST, RESPONSE):
		"""Creates a new todo."""

		return self.get_workspace().make_todo(
			title=self.render_clean_subject(),
			body=self.body,
			REQUEST=REQUEST,
			RESPONSE=RESPONSE)


	def make_event(self, REQUEST, RESPONSE):
		"""Creates a new event."""

		return self.get_workspace().make_event(
			title=self.render_clean_subject(),
			body=self.body,
			REQUEST=REQUEST,
			RESPONSE=RESPONSE)


	def make_note(self, REQUEST, RESPONSE):
		"""Creates a new note."""

		return self.get_workspace().make_note(body=self.body,
				REQUEST=REQUEST,
				RESPONSE=RESPONSE)


	def get_rfc822_message(self):
		"""Returns self."""

		return self


	def use_local_navigation(self):
		"""Returns a boolean."""

		return self.getParentNode().get_product_base_instance().use_local_navigation


	def use_breadcrumbs(self):
		"""Returns a boolean."""

		return self.getParentNode().get_product_base_instance().use_local_navigation


	def mark_as_read(self):
		"""Marks the message as read."""

		self.read = 1
		self.reindex_object()


	def mark_as_unread(self):
		"""Marks the message as unread."""

		self.read = 0
		self.reindex_object()


	def copy_to_folder(self, folder=None):
		"""Copies message to a message container."""

		if folder is not None:

			folder = folder

		else:

			folder = self.get_primary_draft_folder()

		folder.paste(self.copy())

		# We do this so we never get id conflicts.
		# We may get it if it's copied to the same container; FIXME

		new_id = self.get_unique_id()

		folder.manage_renameObject(self.id, new_id)

		message_new = folder[new_id]

		message_new._raw_body = 'No raw body available..'

		return message_new


	def delete(self, delete_from_storage=None, RESPONSE=None):
		"""Moves self to the trash folder."""

		if delete_from_storage:

			self.getParentNode().manage_delObjects(ids=self.id)

		else:

			self.get_gum().get_trash_folder().paste(self.cut())

		self.reindex_object()

		if RESPONSE:

			RESPONSE.redirect(self.getParentNode().relative_url())


	def forward(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for forwarding."""

		if REQUEST.has_key('temporary_folder'):

			draft_folder = REQUEST.resolve_url(REQUEST['temporary_folder'])

		else:

			draft_folder = self.get_primary_draft_folder()

		new_id = str(self.get_unique_id())

		manage_add_rfc822_message(draft_folder, new_id, sender=self.email_address,
							body=self.render_forward_body(),
							subject=self.render_forward_subject())

		new_message = draft_folder[new_id]

		if self.get_attachments():

			for attachment in self.get_attachments():

				new_id = str(self.get_unique_id())

				new_message.manage_add_rfc822_attachment(new_id,
						data=attachment.data,
						filename=attachment.filename,
						content_type=attachment.content_type)
								

		RESPONSE.redirect(new_message.absolute_url())



	def reply(self, REQUEST=None, RESPONSE=None, all=None):
		"""Method that prepares a message for replying."""

		new_id = str(self.get_unique_id())

		new_body = self.render_reply_body()

		if hasattr(self.headers.aq_base, 'references'):

			references = string.split(self.get_header('references'))

		else:

			references = []

		references = []

		headers = {}

		try:

			references.append(self.get_header('message-id'))

		except KeyError:

			# There was no message id..

			pass

		if references:

			headers['references'] = string.join(references, ' ')

		if REQUEST is not None and REQUEST.has_key('draft_folder'):

			# FIXME

			draft_folder = REQUEST.resolve_url(REQUEST['draft_folder'])

		else:

			draft_folder = self.get_primary_draft_folder()

		manage_add_rfc822_message(draft_folder, new_id,
					  sender=draft_folder.email_address,
					  body=new_body,
					  subject=self.render_reply_subject())

		draft_folder[new_id]['to'].manage_add_rfc822_address(
			str(self.get_unique_id()), self.sender, comment='', realname='')

		message_reference = draft_folder[new_id]
		message_reference.set_header('copy-to-folder', '')

		message_reference.references = references[:]

		if all is not None:

			for address_field in self.get_filled_address_fields():
				
				for rfc822_address in self[address_field].objectValues(
					'RFC822Address'):

					message_reference['cc'].manage_add_rfc822_address(
						str(self.get_unique_id()), rfc822_address.address,
						comment=rfc822_address.comment,
						realname=rfc822_address.realname)

			message_reference.newsgroups = self.newsgroups[:]

		message_reference.headers = message_reference.headers

		RESPONSE.redirect(message_reference.absolute_url())


	def reply_all(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for replying."""

		self.reply(REQUEST=REQUEST, RESPONSE=RESPONSE, all=1)


	def render_reply_subject(self):
		"""Method for rendering an appropriate subject upon replies."""

		subject = self.get_header('subject') or self.subject

		if string.lower(subject)[:3] == 're:':

			return subject

		else:

			return 'Re: ' + subject


	def render_forward_subject(self):
		"""Method for rendering an appropriate subject upon forwards."""

		subject=self.headers['subject'].value

		if string.lower(subject)[:4] == 'fwd:':

			return subject

		else:

			return 'Fwd: ' + subject


	def render_html_subject(self):

		subject = html_quote(string.strip(self.render_subject()) or '[No subject]')

		if self.priority > 0:

			if self.priority == 1:

				# Using orange to indicate a normal priority isn't all that

				pass
				#subject = '<span style="color: orange;">%s</span>' % subject

			if self.priority == 2:

				subject = '<span style="color: red;">%s</span>' % subject


		return subject


	def render_subject(self):
		"""Returns the subject."""

		subject = ''

		try:

			subject = string.strip(self.get_header('subject'))

		except KeyError:

			pass

		if not subject:

			try:

				subject = string.strip(self.subject)

			except AttributeError:

				pass

		if not subject:

			subject = '[No subject]'

		return subject


	def render_clean_subject(self):
		"""Returns a 'clean' version of the subject."""

		subject = self.render_subject()

		if string.find(subject, 're:') == 0 or \
			string.find(subject, 'fwd:') == 0:

			return string.split(subject, ':', 1)[1]

		else:

			return subject
		

	def render_reply_body(self):
		"""Method for rendering an appropriate subject upon replies."""

		return '> ' + string.join(string.split(self.body,'\n'), '\n> ')


	def render_forward_body(self):
		"""Quotes the body of the forwarded message."""

		new_body = string.split(self.body, '\n')

		new_body = '>> ' + string.join(new_body, '\n>> ')

		return new_body
	   

	def render_prioritize_widget(self):
		"""Returns an HTML widget for prioritizing."""

		select_widget = """<select name="priority:int">"""

		priority_keys = self.priorities.keys()
		priority_keys.sort()

		for priority in priority_keys:

			if priority == self.priority:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
				(priority, selected, self.priorities[priority])

		return select_widget + """</select>"""


	def render_move_to_widget(self):
		"""Returns an HTML widget for move-to folder selection."""

		select_widget = """<select name="message_container:string">"""

		for message_container in self.get_all_gum_message_containers(type=0):

			select_widget = select_widget + """<option value="%s">%s</option>""" % \
				(message_container.absolute_url(), message_container.get_nested_title())

		return select_widget + "</select>"


	def render_copy_to_header_selection(self):
		"""Returns an HTML widget for copy-to-folder selection."""

		select_widget = """<select name="copy-to-folder:string">"""

		for message_container in [dummy_message_container()] + self.get_all_gum_message_containers(type=1):

			if message_container.relative_url() == 123: #self.headers['copy-to-folder']:

				selected = "selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s" %s>%s</option>""" % \
							(message_container.relative_url(),
								selected,
								message_container.get_nested_title())


		return select_widget + "</select>"


	def render_attachment_count(self):
		"""Returns a string indicating the number of attachments."""

		number_of_attachments = len(self.get_attachments())

		if number_of_attachments:

			return '(%s)' % number_of_attachments

		else:

			return ''


	def render_date(self):
		"""Returns a formatted view of the message date."""

		try:

			date = DateTime.DateTime(self.headers.aq_base['date'].value)
			return self.make_date(date)

		except KeyError:

			return 'No date specified'

		except DateTime.DateTime.DateTimeError, value:

			return self.make_date(self.date_created)
			#return 'Internal error (%s)' % (DateTime.DateTimeError, value)

		except DateTime.DateTime.SyntaxError, value:

			return self.make_date(self.date_created) #return value

		except Exception, value:

			return self.make_date(self.date_created)


	def get_header(self, header_name):
		"""Returns the value of the specified header."""

		return self.headers.get_header(header_name)


	def has_header(self, header_name):
		"""Returns a boolean indicating whether this header is present."""

		return hasattr(self.headers.aq_base, header_name)


	def get_filled_address_fields(self):
		"""Returns address fields that contain one or more addresses."""

		address_fields = []

		for address_field in ['to', 'cc', 'bcc']:

			if len(self[address_field].objectIds('RFC822Address')):

				address_fields.append(address_field)

		return address_fields


	def render_sender(self):
		"""Renders the 'from' address field."""

		return self.sender


	def render_recipients(self):
		"""Returns a whitespace separated list of recipients, suitable for indexing."""

		rendered_address_fields = []

		for address_field in self.get_filled_address_fields():

			rendered_address_fields.append(self.render_address_field(
				field=address_field,
				separator=' '))

		return string.join(rendered_address_fields, ' ')


	def render_newsgroups(self):
		"""Renders the newsgroups header field."""

		return string.join(self.newsgroups, ', ')


	def render_address_field(self, field='to', separator=', '):
		"""Renders the addresses contained in the given field."""

		addresses = map(lambda x: x.address, self[field].objectValues())
		addresses = filter(None, addresses)

		try:

			return string.join(addresses, separator)

		except TypeError:

			return string.join(addresses[0], separator)


	def render_html_priority(self):
		"""Returns an HTML formatted presentation of the priority."""

		text_priority = self.render_priority()

		if self.priority > 0:

			if self.priority == 1:

				return '<span style="color: orange;">%s</span>' % text_priority

			if self.priority == 2:

				return '<span style="color: red;">%s</span>' % text_priority

		return text_priority
		

	def render_priority(self):
		"""Returns a text presentation of the priority."""

		try:

			return self.priorities[self.priority]

		except KeyError:

			# FIXME - implement proper priority conversion when parsing

			return self.priorities[1]


	def move_message(self, REQUEST, RESPONSE):
		"""Moves self to another container."""

		current_message_container_url = self.getParentNode().absolute_url()

		cut_data = self.getParentNode().manage_cutObjects(self.id)

		message_container = REQUEST.resolve_url(REQUEST['message_container'])
		message_container.manage_pasteObjects(cb_copy_data=cut_data)

		self.reindex_object()

		RESPONSE.redirect(current_message_container_url)


	def prioritize(self, REQUEST, RESPONSE):
		"""Prioritizes the message."""

		self.priority = REQUEST['priority']
		self.reindex_object()

		RESPONSE.redirect(self.absolute_url() + '/')


	def update(self, REQUEST=None, RESPONSE=None):
		"""Updates message data."""

		self.body = REQUEST['body']
		self.set_header('subject', REQUEST['subject'])
		self.set_header('copy-to-folder', REQUEST['copy-to-folder'])
		newsgroups = map(string.strip, string.split(REQUEST['newsgroups'], ','))
		self.newsgroups = filter(None, newsgroups)
		self.priority = REQUEST['priority']

		to = filter(None, map(string.strip, string.split(REQUEST['to'], ',')))
		cc = filter(None, map(string.strip, string.split(REQUEST['cc'], ',')))
		bcc = filter(None, map(string.strip, string.split(REQUEST['bcc'], ',')))

		for address_field in ((to,'to'), (cc,'cc'), (bcc,'bcc')):

			existing_addresses = self[address_field[-1]].objectValues()
			request_addresses = address_field[0]

			for rfc822_address in existing_addresses:

				if not rfc822_address.address in address_field[0]:

					self[address_field[-1]].manage_delObjects(rfc822_address.id)

			for address in request_addresses:

				for existing_address in existing_addresses:

					if address == existing_address.address:

						break

				else:

					self[address_field[-1]].manage_add_rfc822_address(
						str(self.get_unique_id()), address)

		if RESPONSE is not None:

			RESPONSE.redirect(self.relative_url() + '/')			


	def use_template(self, RESPONSE):
		"""Copies a template so we can use it."""

		draft_folder = self.get_primary_draft_folder()

		draft_folder.paste(self.copy())

		RESPONSE.redirect(draft_folder.relative_url() + '/' + self.id)


	def save_template(self, REQUEST=None, RESPONSE=None):
		"""Saves a template of this message."""

		# FIXME -- do we need a get_transaction().commit() here?
		# So that the most recent changes to the message body and
		# subject and recipients (etc...) are saved?

		self.update(REQUEST=REQUEST)
		get_transaction().commit()

		template_folder = self.get_primary_template_folder()
		template_folder.paste(self.copy())

		RESPONSE.redirect(self.absolute_url())


	def raw_body(self, RESPONSE):
		"""Returns the original message body."""

		RESPONSE.setHeader('content-type', 'text/plain')

		return self._raw_body


	def send(self, REQUEST=None, RESPONSE=None):
		"""Sends the message."""

		if REQUEST is not None:

			self.update(REQUEST=REQUEST)

		# Do message format magic and such calls here

		if not self.get_filled_address_fields() and not self.newsgroups:

			raise ValueError, 'No recipients or newsgroups specified'

		self.set_header('date', self.ZopeTime().rfc822())

		self.set_header('message-id',
				'<' + str(self.get_unique_id()) + '.' + \
				mimetools.choose_boundary() + '@' + \
				string.split(self.email_address, '@')[-1] + '>')
									   

		try:

			gum = self.get_gum()

			for address in self.get_recipients():

				gum.drop_outgoing_address(address)

		except AttributeError:

			pass

		# We don't want to render the same message twice..
		# Could be very expensive.

		raw_message = self.raw_message()

		if self.get_filled_address_fields():

			res = self.post(self, raw_message=raw_message)

		if filter(None, self.newsgroups):

			news_host = self.get_news_host()
			
			news_host.post(self, raw_message=raw_message)

		# So we don't loose changes made before we send the message.

		get_transaction().commit()

		copy_to_folder = self.unrestrictedTraverse(self.get_header('copy-to-folder'))
		copy_data = self.copy()

		# We have to do it this way to be compatible with all container objects.

		copy_to_folder.manage_pasteObjects(cb_copy_data=copy_data)

		reference = copy_to_folder[self.id]
		reference.readonly = 1

		if self.getParentNode().is_draft_folder():

			self.delete(delete_from_storage=1)

		if RESPONSE:

			# What screen do we return to here?
			#
			# Perhaps a new screen which says
			# message sent?

			RESPONSE.redirect(self.get_gum().absolute_url())


	def add_recipient(self, address_field='to', address='', realname='', comment=''):
		"""Adds a recipient to the given address field."""

		new_id = str(self.get_unique_id())

		self[address_field].manage_add_rfc822_address(new_id, address,
							comment=comment,
							realname=realname)


	def set_header(self, header_name, header_value):
		"""Sets a mail message header."""

		self.headers.set_header(header_name, header_value)


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup to the message."""

		try:

			self.newsgroups.index(newsgroup)
			return

		except ValueError:

			self.newsgroups.append(newsgroup)


	def get_recipients(self):
		"""Returns a list of addresses."""

		recipient_list = []

		for address_field in self.get_filled_address_fields():

			for rfc822_address in self[address_field].objectValues(
				'RFC822Address'):

				recipient_list.append(rfc822_address.address)

		return recipient_list


	def get_attachments(self):
		"""Returns a list of RFC822Attachment objects."""

		return self.objectValues('RFC822Attachment')


	def raw_message(self):
		"""Renders an RFC822 compliant message."""

		rendered_message = cStringIO.StringIO()

		writer = extended_mime_writer.extended_mime_writer(rendered_message)

		writer.addheader('From', self.sender)

		for header_key in self.headers.objectIds():

			if header_key == 'copy-to-folder':

				continue

			writer.addheader(header_key, self.headers[header_key].value)

		writer.addheader('X-Priority', self.render_priority())

		for address_field in self.get_filled_address_fields():

			if address_field != 'bcc':
			
				writer.addheader(
					string.capitalize(address_field),
					mimify.mime_encode_header(
						self.render_address_field(
							field=address_field)))

		if self.newsgroups:

			writer.addheader('Newsgroups', self.render_newsgroups())

		if self.has_header('references'):

			writer.addheader('References', self.get_header(references))
		
		if self.get_attachments():

			writer.addheader('Mime-Version', '1.0')

		writer.addheader('X-Mailer', '%s (%s), %s' % \
			('Zope GUM', self.get_gum_version(), self.get_gum_website()))

		writer.flushheaders()

		if self.get_attachments():

			writer.startmultipartbody('mixed')

			mime_part = writer.nextpart()

			mime_part_writer = mime_part.startbody('text/plain')

			mimetools.encode(cStringIO.StringIO(self.body),
						 mime_part_writer, 'quoted-printable')

			for attachment in self.get_attachments():

				mime_part = writer.nextpart()

				mime_part.addheader('Content-Transfer-Encoding',
							'base64')

				# ?FIXME?
				# Let the attachment encoding decide?

				# FIXME -- update header handling

				tmp_headers = {'content-transfer-encoding':
						'base64'}
				#tmp_headers.update(attachment.headers.make_dictinary())

				mime_part_writer = mime_part.startbody(
						attachment.content_type,
						[('filename', attachment.filename)],
						tmp_headers)

				mime_part_writer.write(attachment.encode())

			else:

				writer.lastpart()

		else:

			writer._fp.write('Content-Transfer-Encoding: quoted-printable\n')

			body_writer = writer.startbody('text/plain', [], # FIXME
						{'Content-Transfer-Encoding':
						'quoted-printable'})

			body = cStringIO.StringIO(self.body)
			body.seek(0)

			mimetools.encode(body, body_writer, 'quoted-printable')

		return rendered_message.getvalue()


	def get_newsgroups(self):
		"""Returns a list of newsgroups."""

		return self.newsgroups


	def get_size(self):
		"""Returns the total size.  Isn't entirely accurate as we don't include headers."""

		size = len(self.body)

		for attachment in self.get_attachments():

			size = size + attachment.get_size()

		return size


	def render_size(self, details=None):
		"""Returns the size in a human-readable format."""

		return utils.format_size(self.get_size(), details=details)


	def get_navigation_title(self):
		"""See below."""

		return self.render_subject() or '[No subject]'


	def name_title_or_id(self):
		"""Returns an appropriate subject, for use in the navigation."""

		return self.get_navigation_title()


	def get_rfc822_message(self):
		"""Returns self."""

		return self


	def manage_afterAdd(self, item, container):
		"""Used for updating message container data."""

		self.message_container_id = container.id
		self.reindex_object()


	'''
	def manage_beforeDelete(self):
		"""Used for removing the message if indexed."""

		self.Catalog.uncatalog_object(self.url())
	'''


	'''
	def index_object(self):
		"""Custom indexing."""

		self.Catalog.catalog_object(self)


	def reindex_object(self):
		"""Custom reindexing."""

		self.Catalog.uncatalog_object(self.url())
		self.Catalog.catalog_object(self)
	'''


	def __init__(self, id, sender, body, subject, message_id='', reply_to='', priority=1,
			readonly=None, references=[]):
		"""Initialize the class instance."""

		Products.BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)

		self.id = id
		self.read = 0
		self.sender = sender
		self.reply_to = reply_to
		self.body = body
		self.message_id = message_id
		self.date_created = DateTime.DateTime()
		self.priority = priority
		self.readonly = readonly
		self.newsgroups = []

		self.manage_add_rfc822_header_container('headers', '')
		self.manage_add_rfc822_address_container('to', title='to')
		self.manage_add_rfc822_address_container('cc', title='cc')
		self.manage_add_rfc822_address_container('bcc', title='bcc')


default__class_init__(rfc822_message)
