###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.2 $'


import MimeWriter
import rfc822_message_parse
import string
import cStringIO
import mimify
import mimetools
import util
import time
# import mxDateTime # We need it to parse non-standard Date headers.


class rfc822_message_base:
	"""Constructs an RFC822 compliant message object."""


	def __init__(self, sender = '', to = [], date='FIXME', cc = [], bcc = [],
				 reply_to = [], newsgroups = [], message_id = '', headers = {},
				 body = '', subject = '', attachments = [],
				 encoding = 'base64', boundary = ''):

		self.newsgroups = newsgroups
		self.sender = sender
		self.reply_to = reply_to
		self.date = date
		self.to = to
		self.cc = cc
		self.bcc = bcc
		self.subject = subject
		self.body = body
		self.attachments = attachments
		self.encoding = 'base64'
		self.headers = headers

		# FIXME

		self.content_type = 'text/plain'

		if message_id:

			self.message_id = message_id
			self.boundary = boundary

		else:  # We assume that the message is outgoing, ?FIXME?

			domain = string.split(self.sender[-1],'@')[-1]

			self.boundary = mimetools.choose_boundary() + '@' + domain
			self.message_id = mimetools.choose_boundary() + '@' + domain


	def add_attachment(self, data, filename='', content_type='', headers={}):
		"""Adds an attachment to the message."""

		self.attachments.append(
			{'data': data, 'filename': filename,
			 'content-type': content_type,
			 'headers': headers})


	def get_attachments(self):
		"""Returns all message attachments."""

		return self.attachments


	def set_header(self, header_name, header_value):
		"""Sets a header to the given value."""

		self.headers[header_name] = header_value


	def get_header(self, header_name):
		"""Returns value of given header."""

		try:

			return self.headers[header_name]

		except:

			return None


	def build_address(self,address_structure):
		"""Builds a '"Name .." (comment) <address>'
		type address header part."""

		return_string = ''

		if address_structure[0]:
				
			return_string = return_string + '"%s" ' % address_structure[0]

		if None: # comment:

			return_string = return_string + '(%s) ' % comment

		return return_string + '<%s>' % address_structure[-1]


	def forward(self):
		"""Prepares the message for forwarding."""

		if self.body:

			self.attachments.insert(0,{'data':self.body,
							'filename':'attachment',
							'media_type':'text',
							'sub_type':'plain',
							'headers':{}})

		self.body = ''

		for i in self.headers.keys():

			self.body = self.body + '%s: %s' % (i, self.headers[i])


	def raw_message(self):
		"""Renders an RFC822 compliant message."""

		rendered_message = cStringIO.StringIO()

		writer = MimeWriter.MimeWriter(rendered_message)

		writer.addheader('From', self.build_address(self.sender))

		writer.addheader('To',string.join(map(self.build_address,self.to),','))

		if self.cc:

			writer.addheader('Cc',string.join(map(
				self.build_address,self.to),','))

		if self.reply_to:
		
			writer.addheader('Reply-To',string.join(map(
				self.build_address,self.to),','))

		if self.newsgroups:

			writer.addheader('Newsgroups',string.join(self.newsgroups,','))

		writer.addheader('Message-ID', self.message_id)

		if len(self.objectValues('RFC822Attachment')):

			writer.addheader('MIME-Version','1.0')

		writer.addheader('Date',self.date)

		# FIXME -- This should be a DateTime object..
		#writer.addheader('Date',self.date.rfc822())
		
		for header in self.headers.keys():

			writer.addheader(header,mimify.mime_encode_header(
				self.headers[header]))

		# We do this to make the recipient headers etc. come before the
		# content-type header.

		writer.flushheaders()

		if self.attachments:

			writer.startmultipartbody('mixed',boundary=self.boundary)

			mime_part = writer.nextpart()

			mime_part_writer = mime_part.startbody('multipart/mixed')

			mimetools.encode(cStringIO.StringIO(self.body),
							 mime_part_writer,'quoted-printable')

			for attachment in self.attachments:

				mime_part = writer.nextpart()

				tmp_headers = {'content-transfer-encoding':attachment.encoding}

				tmp_headers.update(attachment.headers)

				mime_part_writer = mime_part.startbody(
					attachment.content_type(),
					[('filename',attachment.filename)],
					tmp_headers)

				mime_part_writer.write(attachment.encode())

		else:

			# FIXME - we need to take a part first?

			body_writer = writer.startbody(self.content_type,[],0,
							{'Content-Transfer-Encoding':
								'quoted-printable'})

			mimetools.encode(cStringIO.StringIO(self.body,body_writer,
								'quoted-printable'))

		writer.lastpart()

		return rendered_message.getvalue()