###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.3 $

import rfc822_message_parse
import rfc822_message
import rfc822_address
import rfc822_attachment
import cStringIO
import time
import string
import ZODB

priorities = {
		'high':0,
		'normal':1,
		'low':2
	}


def store_message(container, message, readonly=None):
	"""Wrapper so we can make sure the message is saved."""

	while 1:

		try:


			message_reference = _store_message(container, message, readonly=readonly)
			get_transaction().commit(1)
			#container._p_jar._resetCache()
			break

		except ZODB.POSException.ConflictError:

			container._p_jar.sync()

	#container.Catalog.catalog_object(message_reference)
	message_reference.message_container_id = container.id
	message_reference.index_object()

	return message_reference


def _store_message(container, message, readonly=None):
	"""A method that adds an rfc822_message object
	to the container, and possibly address fields.
	It may also add attachments."""

	message_id = str(container.get_unique_id())

	if message.headers.has_key('subject'):

		if not string.strip(message.headers['subject']):

			message.subject = '[No subject]'

	else:

		message.subject = '[No subject]'

	if message.headers.has_key('x-priority'):

		try:

			priority = int(message.get_header('x-priority'))

		except ValueError:

			priority = 1

	else:

		priority = 1

	rfc822_message.manage_add_rfc822_message(container, message_id,
						sender=message.sender[-1],
						body=message.body,
						subject=message.subject,
						reply_to=message.reply_to,
						priority=priority,
						readonly=readonly)

	message_reference = container[message_id]

	for header_name in message.headers.keys():

		message_reference.set_header(header_name, message.headers[header_name])

	if message.attachments:

		for attachment in message.attachments:

			message_reference.manage_add_rfc822_attachment(
				str(container.get_unique_id()),
				data=attachment.data,
				headers=attachment.headers,
				filename=attachment.filename,
				content_type=attachment.media_type + \
					'/' + attachment.sub_type)


	if message.newsgroups:

		message_reference.newsgroups.extend(message.newsgroups)

	for recipient in message.to:

		message_reference['to'].manage_add_rfc822_address(
			str(container.get_unique_id()),
			recipient[-1],
			comment='comment',
			realname=recipient[0])

	for recipient in message.cc:

		message_reference['cc'].manage_add_rfc822_address(
			str(container.get_unique_id()),
			recipient[-1],
			comment='comment',
			realname=recipient[0])

	return message_reference


def store_unparsed_message(container, unparsed_rfc822_message, readonly=None):
	"""This method lets us parse a message before storing it."""

	if type(unparsed_rfc822_message) is type(''):

		unparsed_rfc822_message = my_string_io(unparsed_rfc822_message)

	message = rfc822_message_parse.rfc822_message_parse(
		unparsed_rfc822_message)

	store_message(container, message, readonly=readonly)


class my_string_io:
	"""This class represents a wrapper for a cStringIO.StringIO instance,
	giving the ability to use the readlines method."""

	# Should we use a __getattr__ wrapper instead of per-method wrapping?
	# This will prevent breakage of future methods StringIO may have...

	# This isn't needed in Python 2.0 as readlines is implemented.

	def __init__(self, data=''):

		self._fp = cStringIO.StringIO(data)


	def close(self):

		return self._fp.close()


	def flush(self):

		return self._fp.flush()


	def isatty(self):

		return self._fp.isatty()


	def fileno(self):

		return self._fp.fileno()


	def read(self, size=None):

		if size:

			return self._fp.read(size)

		return self._fp.read()


	def readline(self):

		return self._fp.readline()


	def readlines(self):

		return_list = []

		tmp = None

		while 1:

			tmp = self._fp.readline()

			if tmp:

				return_list.append(tmp)

			else:

				break

		return return_list


	def seek(self, offset, whence=None):

		if whence:

			return self._fp.seek(offset, whence)

		return self._fp.seek(offset)


	def tell(self):

		return self._fp.tell()


	def truncate(self, size=None):

		if size:
			
			return self._fp.truncate(size)

		return self._fp.truncate()


	def write(self, _string):

		return self._fp.write(_string)


	def writelines(self, lines):

		return self._fp.writelines(lines)
