__doc__='''Spellchecker'''
__version__='1.0'

from Globals import HTMLFile, MessageDialog, package_home
import OFS.SimpleItem, Acquisition, AccessControl.Role
from BoboPOS import Persistence
from OFS.Document import Document
import os, time, string

def manage_addSpellchecker(self,id, REQUEST=None):
	"""Adds Spell Checker Object"""
	i=Spellchecker(id)
	self._setObject(id,i)
	return self.manage_main(self,REQUEST)

manage_addSpellcheckerForm=HTMLFile('spellerAdd', globals())

class Spellchecker(
	OFS.SimpleItem.Item,
	Persistence.Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	):
	"""Spellchecker object"""
	# Meta_type is useful for for looking up objects in DTML
	# objectItems(['EventEntry']) will return
	# a list of guest book entries
	meta_type='Spellchecker'

	# 'Borrow' the normal document icon ;)
	icon = 'p_/doc'

	manage_main=HTMLFile('manage', globals())
	# Permission information for Guestbookitems is assumed to be the
	# same as for the enclosing GuestBook. We do nothing here, but
	# permission information is acquired from the enclosing Guestbook,
	# as we're subclassing Acquisition.Implicit

	def __init__(self, id):
		"""Create an entry"""
		self.id	  = id
		
	# index_html for guest book entries
	# Not accessible at the moment (except by URL hacking)
	# Could be made to do something useful!
	index_html  = HTMLFile('index', globals())



	def get_next_word(self,word_count=0,pos=0,text="",ignore_list=[]):
		""" Get next misspelled word. """

		import re
		import os
		import string


		def run_checker(word=""):
			"""Run spell checker on given word"""

			import os
			import string

			ispell = os.popen("echo " + word + " | ispell -a")
			ispell.readline()
			sentence = ispell.readline()

			if (sentence[0] == '&'):
				parts = string.split(sentence, ": ")
				gluedwords = parts[1]
				words = string.split(gluedwords,", ")
				length = len(words)
				words[length-1] = words[length-1][0:len(words[length-1]) - 1]
				return [word] + words

			elif (sentence[0] == '#') or (sentence[0] == '?'):
				return [word]
			else:
				return []



		parsed_words = re.split('\W+',text) #Get list of words  
		if not len(parsed_words):
			return[[],word_count,pos,text]

		#Run through the correct words 
		suggestions=[]
		while not suggestions:
			if word_count>=len(parsed_words):				  #If end of text.
				return[[],word_count,pos,text]	
			if (not len(parsed_words[word_count])):
				word_count=word_count+1
			if word_count>=len(parsed_words):				  #If end of text.
				return[[],word_count,pos,text]	
			current_word=parsed_words[word_count]
			suggestions=run_checker(current_word)
			if suggestions!=[]:
				for x in ignore_list:
					if x==current_word:
						suggestions=[]
						break		
			pos=string.find(text,current_word,pos)+len(current_word)  #pos becomes end of word
			word_count=word_count+1								   #count next word to check
			if word_count>len(parsed_words):						  #If end of text.
				return[[],word_count,pos,text]							 

		#Return suggestiong for misspelled word   
		return[suggestions,word_count,pos,text]		   


	def replace_word(self,text,new_word,old_word,replace_all,word_count,pos,ignore_list):
		"""Replace and call get_next_word"""

		import os
		import string
		import re

		start_pos=pos-len(old_word)
		if(replace_all):
			i=0
			words_and_delimiters=re.split('(\W+)',text[start_pos:])
			for x in words_and_delimiters[:]:
				if x==old_word:
					words_and_delimiters[i]=new_word
				i=i+1
			text=text[:start_pos]+string.join(words_and_delimiters,'')
		else:
			text=text[:start_pos]+string.replace(text[start_pos:],old_word,new_word,1)
		pos=start_pos+len(new_word)	
		word_count = word_count+len(re.split('\W+',new_word))-1 #in case of several new words
		return_val=self.get_next_word(word_count,pos,text,ignore_list)
		return return_val

	def add_word(self,word):
		"""Add a word to the ispell personal dictionary"""
		import sys
		import os

		dictionary = open(os.environ.data['HOME'] + "/.ispell_english", "a")
		dictionary.write(word + "\n")
		dictionary.close()
		os.system("sort -f -i ~/.ispell_english -o ~/.ispell_english")

	def divide(self,gluedwords):			#The gluedwords are separated by ","
		"""unglue words separated by ,  """

		import string

		if not len(gluedwords):
			return []
		return string.split(gluedwords, ",")


	def highlight(self,text,oldword,pos):
		"""Hilights."""
		return text[:pos-len(oldword)]+"<font type=\"bold\" color=\"red\">"+oldword+"</font>"+text[pos:]




