###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $

__doc__ = 'Value handler'
__version__ = '$Revision 1.0 $'

import Globals
import ZPublisher
import string
import convert_integer
import convert_float
import convert_required
import convert_empty_as_none
import convert_list
import convert_text
import convert_string
import convert_boolean
import convert_email
import convert_tokens
import convert_lines
import convert_year
import compose_time
import compose_date
import compose_datetime

value_handlers = {}

message_dialogue = Globals.HTMLFile('dtml/message_dialogue', globals())

http_request_http_request_process_inputs = ZPublisher.HTTPRequest.HTTPRequest.processInputs

def register_value_handler(name, module):
	"""This let's us add value handlers from other modules."""

	value_handlers[name] = module


register_value_handler('integer', convert_integer)
# Don't like it one bit (pun intended :-).
register_value_handler('int', convert_integer)
register_value_handler('required', convert_required)
register_value_handler('float', convert_float)
register_value_handler('list', convert_list)
register_value_handler('text', convert_text)
register_value_handler('string', convert_string)
register_value_handler('boolean', convert_boolean)
register_value_handler('emtpy_as_none', convert_empty_as_none)
register_value_handler('email', convert_email)
register_value_handler('tokens', convert_tokens)
register_value_handler('lines', convert_lines)
register_value_handler('year', convert_year)
register_value_handler('datetime', compose_datetime)
register_value_handler('date', compose_date)
register_value_handler('time', compose_time)


def process_inputs(request):
	"""Method that hooks itself onto the standard form data processing."""

	# We don't want to disturb anybody

	http_request_http_request_process_inputs(request)

	ignore_conversion_error = request.form.has_key('__ignore_conversion_error')
	ignore_unknown_value_handler = request.form.has_key('__ignore_unknown_value_handler')

	# How can we can ahold of the object being published?
	# Seems we have to do something that they do in the ZPublisher.Publish module..
	# Hmmmm.... =]

	published_object = None

	string_find = string.find
	string_split = string.split

	request.value_handler_exception_stack = []
	composite_values = []
	form_update = {}

	for key in request.form.keys():

		if string_find(key, ';') > -1:

			splits = string_split(key, ';')

			field_name = splits[0]

			if field_name == '__composite_value':

				composite_values.append(splits[1:])
				continue

			if 'ignore' in splits:

				continue

			for value_handler in splits[1:]:

				value = request.form[key]

				if value_handler in value_handlers.keys():

					value_handler = value_handlers[value_handler]

					try:

						# So we can check several conditions

						try:

							value = form_update[field_name]

						except KeyError:

							pass

						form_update[field_name] = value_handler.convert(value)

					except ValueError, exception_value:

						if ignore_conversion_error:

							form_update[field_name] = value
							continue

						request.value_handler_exception_stack.append(
							{
								'field_name':field_name,
								'value':exception_value,
								'help':value_handler.help
							})

						# We need to add some logic so that the user
						# doesn't have to go back, fill in the wrong value
						# and then go forward again, to then go back,
						# i.e. that the value needs to be integer and
						# that it is required, etc...

						break

				else:

					if ignore_unknown_value_handler:

						form_update[key] = value
						continue

					raise SyntaxError, 'Unknown value checker/converter: %s' % value_handler

	for composite_value in composite_values:

		try:

			composite_name, composite_type = composite_value

		except TypeError:

			raise SyntaxError, 'Malformed construction of composite indicator: %s' % composite_value

		try:

			value_handler = value_handlers[composite_type]

		except KeyError:

			raise SyntaxError, 'Unknown composite type: %s' % composite_type

		try:

			form_update[composite_name] = value_handler.compose(composite_name, form_update)

		except ValueError:

			if ignore_conversion_error:

				form_update[composite_name] = value
				continue

			else:

				request.value_handler_exception_stack.append(
						{
							'field_name':composite_name,
							'value':'No value',
							'help':value_handler.help
						})

	for key, value in form_update.items():

		request.set(key, value)

	if request.value_handler_exception_stack:

		if hasattr(published_object, 'standard_value_error_message'):

			published_object.standard_value_error_message(None, REQUEST=request, RESPONSE=request.RESPONSE)

		else:

			request.RESPONSE.write(message_dialogue(None, REQUEST=request, RESPONSE=request.RESPONSE))


ZPublisher.HTTPRequest.HTTPRequest.processInputs = process_inputs
