###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.8 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

from OFS import ObjectManager
from OFS.CopySupport import _cb_decode
from utils import html_quote # To get readable debug info in HTML


class object_manager(ObjectManager.ObjectManager):
    """Used for grouping object management related methods and attributes."""

    object_sheet = ()
    meta_buttons = ()
    meta_types = ()
    copy_support = None

    def get_meta_buttons(self):
        """Returns the meta buttons, if any."""
        return self.meta_buttons

    def get_object_listing(self, meta_type, status=0):
        """Returns a set of objects."""
        objects = []
        object_sheet = self.get_object_sheet(meta_type)
        for object in self.get_objects_by_meta_type(meta_type, status=status):
            object_data = object.get_object_sheet_data()
            objects.append({
                'id':object.id,
                'absolute_url':object.absolute_url(),
                'data':object_data}
            )
        # We need more optimized stuff here
        objects.sort(lambda x,y: cmp(y['id'], x['id']))
        return objects

    def get_number_of_objects(self, meta_type, status=0, **kw):
        """Returns the number of objects with the given meta_type."""
        return len(self.get_objects_by_meta_type(meta_type, status=status))

    def get_objects_by_meta_type(self, meta_type, status=0):
        """Returns a listing of objects."""
        if hasattr(self.aq_base, '_virtual_folder_requirements'):
            if hasattr(self.aq_base, 'catalog_expression'):
                catalog = eval('self.%s' % self.catalog_expression)
            else:
                catalog = self.Catalog
            requirements = {'meta_type':meta_type}
            for requirement in self._virtual_folder_requirements:
                key = requirement[0]
                value = eval(requirement[1])
                requirements[key] = value
            brains = catalog.searchResults(requirements)
            return map(catalog.getobject, map(lambda x: x.getRID(), brains))
        else:
            # Implement
            return self.objectValues(meta_type)


    def manage_delete(self, ids=[], REQUEST=None, RESPONSE=None):
        """Deletes given ids."""
        if type(ids) is type(''):
            ids = [ids]
        self.manage_delObjects(ids)
        if REQUEST is not None:
            if REQUEST.has_key('__referrer'):
                RESPONSE.redirect(REQUEST['__referrer'])
            else:
                RESPONSE.redirect(self.absolute_url())

    def manage_delete_all(self, REQUEST=None, RESPONSE=None):
        """Method that deletes all messages in folder."""
        if REQUEST is not None:
            if REQUEST.has_key('confirm') and \
                REQUEST['confirm']:
                pass
            else:
                RESPONSE.redirect(self.absolute_url())
                return
        self.manage_delete(ids=self.objectIds())
        RESPONSE.redirect(self.absolute_url())

    def base_copy(self, ids=[], REQUEST=None, RESPONSE=None):
        """Wrapper."""
        self.manage_copyObjects(ids=ids, REQUEST=REQUEST, RESPONSE=RESPONSE)
        RESPONSE.redirect(self.absolute_url())

    def base_paste(self, copy_data, REQUEST=None):
        """Wrapper."""
        return self.manage_pasteObjects(cb_copy_data=copy_data, REQUEST=REQUEST)

    def get_copyable_meta_types(self):
        """Returns a list of copyable meta types."""
        return map(lambda x: x['name'], self.meta_types)

    def decode_copy_string(self, copy_string):
        """Decodes the copy string."""
        return _cb_decode(copy_string)

    def accepts_meta_type(self, meta_type):
        """Returns a truth value if the given meta type is accepted."""
        try:
            return self.get_copyable_meta_types().index(meta_type)
        except ValueError:
            return None

    def decode_copy_data(self, copy_data):
        """Decodes the copy data and returns the object references."""
        objects = []
        # FIXME - we need to return the actual objects here
        for object_copy_data in self.decode_copy_string(copy_data):
            objects.append(object_copy_data)
        return objects

    def get_buttons_for_meta_type(self, meta_type):
        """Returns a set of buttons for the given meta type."""
        buttons = []
        for button in self.meta_buttons:
            if button['meta_type'] == meta_type:
                buttons.append(button)
        return buttons

    def manage_add(self, meta_type=None, REQUEST=None, RESPONSE=None, id=None):
        """Adds a new instance of the specified meta type or returns the relevant adding page."""
        for meta_type_ in self.meta_types:
            if meta_type_['name'] == meta_type:
                break
        else:
            raise 'UnknownSubclass', REQUEST
        klass = self.get_class_by_meta_type(meta_type=meta_type_['name'])
        if hasattr(self, 'manage_add_%s_form' % klass.__name__):
            if RESPONSE is not None:
                RESPONSE.redirect(self.absolute_url() + '/manage_add_%s_form' % klass.__name__)
            else:
                raise 'NoResponse'
            return
        elif hasattr(self, 'manage_add_' + klass.__name__):
            if RESPONSE is not None:
                RESPONSE.redirect(self.absolute_url() + '/manage_add_' + klass.__name__)
                return
        if id is None:
            id = self.get_unique_id()
        try:
            self._setObject(id, klass(id))
        except TypeError, value:
            raise TypeError, html_quote(str((value, str(klass))))
        if RESPONSE is not None:
            RESPONSE.redirect(self.absolute_url() + '/' + id)
        else:
            return getattr(self, id)
