###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.10 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

import utils
from Globals import DTMLFile
import Products
import sys

class product_base_item:

    #standard_html_header = DTMLFile('dtml/standard_html_header', globals())
    #standard_html_footer = DTMLFile('dtml/standard_html_footer', globals())
    #standard_css = DTMLFile('dtml/standard_css', globals())

    default_installation = utils.default_installation

    def get_catalog(self):
        """Returns the products' base catalog."""
        return self.Catalog

    def get_preferences_url(self):
        """Returns a preferences URL."""
        return self.relative_url() + '/base_edit'

    def get_product_base_instance(self):
        """Returns self."""
        return self

    def create_catalog_and_indexes(self, create_catalog=1, catalog_name='Catalog'):
        """Creates a ZCatalog instance and adds indexes for the given product."""
        created_indexes = []
        created_columns = []
        if create_catalog:
            self._setObject(catalog_name, Products.ZCatalog.ZCatalog.ZCatalog(catalog_name, ''))
        for meta_type, index_sheets in self.get_index_sheets_for_product(self.meta_type):
            for index_set in index_sheets:
                index_type = index_set[0]
                if index_type == 'Column':
                    for column in index_set[1]:
                        try:
                            self[catalog_name].manage_addColumn(column)
                            created_columns.append(column)
                        except:
                            type, value, traceback = sys.exc_info()
                            if not type == 'Column Exists':
                                raise
                else:
                    for index in index_set[1]:
                        try:
                            self[catalog_name].manage_addIndex(index, index_type)
                            created_indexes.append((index, index_type))
                        except:
                            type, value, traceback = sys.exc_info()
                            if type == 'Index Exists':
                                real_catalog = self[catalog_name]._catalog
                                existing_index = real_catalog.indexes[index]
                                if existing_index.meta_type == 'Field Index' and \
                                    index_type == 'FieldIndex':
                                    pass
                                elif existing_index.meta_type == 'Text Index' and \
                                    index_type == 'TextIndex':
                                    pass
                                elif existing_index.meta_type == 'Keyword Index' and \
                                    index_type == 'KeywordIndex':
                                    pass
                                else:
                                    raise 'Conflicting indexes', \
                                    'The indexes %s (%s) and %s (%s) conflict' % \
                                    (index, index_type, index, existing_index.meta_type)
                            else:
                                raise
        if created_columns or created_indexes:
            return (created_columns, created_indexes)
        else:
            return ()

    def update_catalog(self, catalog_name='Catalog', REQUEST=None):
        """Adds new indexes and columns and updates the catalog if necessary."""
        if self.create_catalog_and_indexes(create_catalog=0, catalog_name=catalog_name):
            if REQUEST is not None:
                catalog = getattr(self, catalog_name)
                REQUEST.RESPONSE.redirect(catalog.absolute_url() +
                    '/manage_catalogReindex')
            return 1
        else:
            if REQUEST is not None:
                REQUEST.RESPONSE.redirect(self.absolute_url())
            else:
                return 0

    def update_instances(self):
        """Updates all instances to reflect the new class definitions."""
        # We could probably make this a bit smarter so it looks at the
        # various class files and such, any takers?
        meta_types_updated = []
        catalog = self.get_catalog()
        meta_types = catalog.uniqueValuesFor('meta_type')
        for meta_type in meta_types:
            try:
                brains = catalog(meta_type=meta_type)
                if not brains[0].getObject().check_property_integrity():
                    meta_types_updated.append(meta_type)
                    for brain in brains:
                        brain.getObject().initialize_properties(
                            keep_old_values=1)
            except IndexError:
                # No instances for meta_type
                pass
        # FIXME
        return str(meta_types_updated)

    def update(self, REQUEST=None):
        """Updates all instances within the product instance."""
        self.update_instances()
        if self.update_catalog(REQUEST=REQUEST):
            return # If the catalog was updated, we'll need to
                   # reindex, so we don't redirect
        else:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    def get_index_sheets_for_product(self, product_name):
        """Returns the index sheets for the product."""
        index_sheets = []
        for meta_type, sheets in Products.product_registry[product_name].items():
            if sheets.has_key('index_sheet'):
                index_sheets.append(meta_type, sheets['index_sheet'])
        return index_sheets

    def get_version(self):
        """Returns the product version."""
        try:
            file = open('%s/Products/%s/version.txt' %
                (SOFTWARE_HOME, self.product_name))
            return file.read()
        except IOError:
            return 'Failed reading version.txt, no version available'

    def render_date(self, date=None, format="%Y-%m-%d %H:%M"):
        """Returns an instance specific format of the time."""
        # Hack to circumvent ZCatalog exception, FIXME
        if date == None:
            return None
        if not format:
            format = "%Y-%m%d %H:%M"
        try:
            return date().strftime(format)
        except AttributeError:
            return date.strftime(format)
    make_date = render_date
