###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.1 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

import Globals
from Products import BTreeFolder
import Acquisition
import DateTime
import string
import base_item


def add_history(instance):
	"""Adds a history object to an object."""

	instance._setObject('history', history_folder())


class history_folder(
		Globals.Persistent,
		BTreeFolder.BTreeFolder.BTreeFolder,
		Acquisition.Implicit,
		base_item):
	"""Used to keep track of object history."""

	meta_type = 'History folder'
	title = 'History'

	meta_types = (
		{
			'name':'History entry',
			'action':''
		},
	)

	_properties = ()
	property_sheets = ()

	index_html = Globals.HTMLFile('dtml/property_history_folder_index', globals())

	def __init__(self):
		"""Initializes the property history."""

		self.id = 'history'
		BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)


	def add_entry(self, author, property_sheet_name, changed_properties):
		"""Adds a change entry to the property sheet history."""

		new_id = string.replace(str(float(DateTime.DateTime())), '.', '_')

		self._setObject(new_id, history_entry(new_id,
			author, property_sheet_name, changed_properties))


Globals.default__class_init__(history_folder)


class history_entry(
		Globals.Persistent,
		BTreeFolder.BTreeFolder.BTreeFolder,
		Acquisition.Implicit,
		base_item):
	"""Used to keep track of a property sheet's history."""

	title = meta_type = 'History entry'

	index_html = Globals.HTMLFile('dtml/property_history_index', globals())

	def get_changed_properties_listing(self):
		"""Returns a simple listing of the changed properties."""

		changed_properties = []

		for property in self.changed_properties:

			get_property_display = self.getParentNode().get_property_display
			property_data = self.getParentNode().get_property(property[0])

			tmp = {
				'rendered_id':property_data['rendered_id'],
				'old_value':get_property_display(property[0], property_value=property[1]),
				'changed_value':get_property_display(property[0], property_value=property[2]),
				'current_value':get_property_display(property[0]),
			}

			changed_properties.append(tmp)

		return changed_properties


	def render_property_sheet(self):
		"""Returns a string with the property sheet name."""

		return str(self.property_sheet_name)


	def get_previous_history_entry_id(self):
		"""Returns the previous history entry id."""

		ids = self.getParentNode().objectIds()
		ids.sort()
		ids.reverse()

		try:

			index = ids.index(self.id)

			if index == 0:

				return None

			return ids[index-1]

		except ValueError:

			return None


	def get_next_history_entry_id(self):
		"""Returns the next history entry id."""

		ids = self.getParentNode().objectIds()
		ids.sort()
		ids.reverse()

		try:

			index = ids.index(self.id)
			return ids[index+1]

		except IndexError, ValueError:

			return None


	def __init__(self, id, author, property_sheet_name, changed_properties):
		"""Initializes the property history."""

		BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)
		self.id = id
		self.creator = self.author = author
		self.property_sheet_name = property_sheet_name
		self.changed_properties = changed_properties
		self.date_created = DateTime.DateTime()


Globals.default__class_init__(history_entry)
