###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.3 $

__doc__ = "Documentation string"
__version__ = "0.1.0"


import DateTime
import string
import types
import html_widgets
import composite_widgets
import property_display
import operator
import property_history


class property_manager:
	"""Class used to manage and display properties."""

	property_sheets = ()
	_properties = ()

	default_datetime = DateTime.DateTime('2001/01/01')
	default_datetime_start = DateTime.DateTime('2001/01/01')
	default_datetime_end = DateTime.DateTime('2010/01/01')

	# This should perhaps be moved to base_item

	def get_select_options(self, **kw):
		"""Returns a sequence of options based on a ZCatalog query, used
		in select and radio_collections."""

		options = (('','Select.. (not defined)'),)

		for catalog_brain in self.Catalog.searchResults(kw, sort_on='get_navigation_title'):

			options = options + ((catalog_brain.id, catalog_brain.get_navigation_title),)

		return options

	# This should perhaps be moved to base_item

	def get_checkbox_options(self, **kw):
		"""Returns a sequence options based on a ZCatalog query,
		used in checkbox collections."""

		options = ()

		for catalog_brain in self.Catalog.searchResults(kw, sort_on='get_navigation_title'):

			options = options + ((catalog_brain.id, catalog_brain.get_navigation_title),)

		return options


	def get_property_sheet_names(self):
		"""Returns a list of property sheet names."""

		return map(lambda x: x[0], self.property_sheets)


	def get_property_sheet(self, property_sheet_name):
		"""Returns the relevant property sheet.  May raise a ValueError."""

		for property_sheet in self.property_sheets:

			if property_sheet[0] == property_sheet_name:

				return property_sheet[1]

		else:

			raise ValueError


	def get_property(self, property_id):
		"""Returns property data for given property."""

		for property in self._properties:

			if property['id'] == property_id:

				property['value'] = getattr(self, property_id)

				if not property.has_key('rendered_id'):

					property['rendered_id'] = string.capitalize(string.replace(property_id, '_', ' '))

				return property

		else:

			raise ValueError, 'Property not found: %s' % property_id


	def get_property_widgets_listing(self, property_sheet=None):
		"""Returns a list of mappings, suitable for iterations."""

		properties = []
		properties_mapping = self.get_property_widgets_mapping(property_sheet=property_sheet)

		for property_id in properties_mapping['properties_order_']:

			properties.append(properties_mapping[property_id])

		return properties


	def get_property_widgets_mapping(self, property_sheet=None):
		"""Returns the widgets needed to represent and manipulate properties with widgets."""

		widgets = {'properties_order_':[]}

		properties = []

		if property_sheet is not None:

			property_sheet = self.get_property_sheet(property_sheet)

			for property_id in property_sheet:

				properties.append(self.get_property(property_id))

		else:

			properties = self._properties

		for property in properties:

			property_id = property['id']

			result = self.get_property_widget(property_id)

			widget = result['widget']
			rendered_id = result['rendered_id'] 

			# Ok, ok, this is an ugly hack

			if property.has_key('link'):

				primary_widget = widgets[property['link']]

				if primary_widget['widget'].type != 'simple':

					widgets[property['link']] = {
						'widget':composite_widgets.simple(),
						'rendered_id':primary_widget['rendered_id']+'(linked)'
					}

					widgets[property['link']]['widget'].add_widget(primary_widget['widget'])

				widgets[property['link']]['widget'].add_widget(widget)

			else:

				widgets['properties_order_'].append(property_id)
				widgets[property_id] = \
					{
						'widget':widget,
						'rendered_id':rendered_id
					}

		return widgets


	def get_property_widget(self, property_id, property_value=None):
		"""Returns a widget for manipulating the given property."""

		string_capwords = string.capwords
		string_capitalize = string.capitalize
		string_replace = string.replace

		use_html = 1

		if use_html:

			textarea = html_widgets.textarea
			select = html_widgets.select
			select_option = html_widgets.select_option
			radio_collection = html_widgets.radio_collection
			radio = html_widgets.radio
			checkbox_collection = html_widgets.checkbox_collection
			checkbox = html_widgets.checkbox
			text = html_widgets.text

			composite_time = composite_widgets.time_widget
			composite_date = composite_widgets.date_widget
			composite_datetime = composite_widgets.datetime_widget
			composite_simple = composite_widgets.simple

			make_select = html_widgets.make_select
			make_checkbox_collection = html_widgets.make_checkbox_collection

		operator_truth = operator.truth
		
		property = self.get_property(property_id)

		if property_value is not None:

			pass

		else:

			property_value = property['value']

		property_type = property['type']
		property_rendered_id = property['rendered_id']
		# FIXME - need to fix this at some point (non-markup)
		property_widget_name = property_id + ';' + property_type

		if property.has_key('widget_type'):

			widget_type = property['widget_type']

			if widget_type == 'select':

				select_widget = property['data']

				try:

					widget_multiple = property['multiple']

				except KeyError:

					widget_multiple = 0

			else:

				select_widget = ""

			if widget_type == 'checkbox_collection':

				checkbox_collection_widget = property['data']

			else:

				checkbox_collection_widget = ""

		else:

			select_widget, checkbox_collection_widget = ('', '')

		if select_widget:

			select_widget = getattr(self, select_widget)

			if callable(select_widget):

				options = select_widget()

			else:

				options = select_widget

			# Mark

			widget = make_select(name=property_widget_name, multiple=widget_multiple,
						options=options, selected_values=property_value)



		elif checkbox_collection_widget:

			checkbox_collection_widget = getattr(self, checkbox_collection_widget)

			if callable(checkbox_collection_widget):

				options = checkbox_collection_widget()

			else:

				options = checkbox_collection_widget

			widget = make_checkbox_collection(name=property_widget_name, options=options,
						selected_values=property_value)


		elif property_type == 'text':

			widget = textarea(name=property_widget_name, value=property_value,
						rows=self.textarea_input_rows,
						cols=self.textarea_input_cols)

		elif property_type == 'lines':

			widget = textarea(name=property_widget_name,
					value=string.join(property_value, '\n'),
					rows=self.textarea_input_rows,
					cols=self.textarea_input_cols)

		elif property_type == 'boolean':

			if property.has_key('data'):

				boolean_widget = getattr(self, property['data'])

			else:

				boolean_widget = self.yes_no_options

			if callable(boolean_widget):

				options = boolean_widget()

			else:

				options = boolean_widget

			widget = radio_collection(name=property_widget_name)

			property_value = operator_truth(property_value)

			for (option_key, option_value) in options:

				option_key = operator_truth(option_key)

				if option_key == property_value:

					checked = 1

				else:

					checked = 0

				widget.add_option(
					value=option_key,
					displayed_value=option_value,
					checked=checked)

		elif property_type == 'date':

			widget = composite_date(name=property_id, datetime=property_value)

		elif property_type == 'datetime':

			widget = composite_datetime(name=property_id, datetime=property_value)

		elif property_type == 'time':

			widget = composite_time(name=property_id, seconds=property_value)

		else:

			if property_type in ('int', 'float'):

				size = 10

			else:

				size = None

			widget = text(name=property_widget_name, value=property_value, size=size)

		widget.rendered_id = property_rendered_id
			
		return {'widget':widget, 'rendered_id':property_rendered_id}
		

	def get_property_listing(self, property_sheet=None):
		"""Returns a list of mappings, suitable for iterations."""

		properties = []
		properties_mapping = self.get_property_mapping(property_sheet=property_sheet)

		for property_id in properties_mapping['properties_order_']:

			properties.append(properties_mapping[property_id])

		return properties


	def get_property_mapping(self, property_sheet=None):
		"""Returns a mapping of objects to represent the data."""

		properties_mapping = {'properties_order_':[]}

		properties = []

		if property_sheet is not None:

			property_sheet = self.get_property_sheet(property_sheet)

			for property_id in property_sheet:

				properties.append(self.get_property(property_id))

		else:

			properties = self._properties

		for property in properties:

			property_id = property['id']
			properties_mapping['properties_order_'].append(property_id)

			result = self.get_property_display(property_id)

			display = result['display']
			rendered_id = result['rendered_id'] 

			properties_mapping[property_id] = \
				{
					'display':display,
					'rendered_id':rendered_id
				}

		return properties_mapping


	def get_property_display(self, property_id, property_value=None):
		"""Returns a display for the given property."""

		property = self.get_property(property_id)

		if property_value is None:

			property_value = property['value']

		try:

			widget_type = property['widget_type']

		except KeyError:

			widget_type = ""

		if widget_type == 'select':

			data = getattr(self, property['data'])
			multiple = property.has_key('multiple')

			if callable(data):

				data = data()

			if multiple:

				sequence = []

				for option in data:

					if option[0] in property_value:

						sequence.append(option[1])

				display = property_display.sequence(sequence)

			else:

				for option in data:

					if option[0] == property_value:

						display = property_display.string_(option[1])
						break

				else:

					display = property_display.string_('Not defined')
				
		elif widget_type == 'checkbox_collection':

			data = getattr(self, property['data'])
			sequence = []

			if callable(data):

				data = data()

			for option in data:

				if option[0] in property_value:

					sequence.append(option[1])

			display = property_display.sequence(sequence)

		elif property['type'] == 'boolean':

			property_value = operator.truth(property_value)

			if property.has_key('data'):

				data = getattr(self, property['data'])

			else:

				data = self.yes_no_options

			if callable(data):

				data = data()

			data = map(lambda x: (operator.truth(x[0]), x[1]), data)

			for option in data:

				if option[0] == property_value:

					display = property_display.boolean(option[1])
					break

		elif property['type'] in ('lines', 'list', 'tokens'):

			display = property_display.sequence(property_value)


		elif property['type'] == 'text':

			display = property_display.text(property_value)

		elif property['type'] == 'string':

			display = property_display.string_(property_value)

		else:

			display = str(property_value)

		try:

			return {
					'display':display,
					'rendered_id':property['rendered_id']
			}

		except:

			raise property_id


	def initialize_properties(self,
			string_default='', int_default=0,
			float_default=0.0, text_default='',
			boolean_default=0, lines_default=[],
			list_default=[], tokens_default=[],
			date_default=None, datetime_default=None,
			time_default=43200):
		"""Iterates over all known properties and initializes them."""

		date_default = date_default or self.default_datetime
		datetime_default = datetime_default or self.default_datetime

		self.date_created = DateTime.DateTime()
		property_history.add_history(self)

		for property in self._properties:

			if property.has_key('default_value'):

				value = property['default_value']

			else:

				property_type = property['type']

				if property_type == 'string':

					value = string_default

				elif property_type == 'int':

					value = int_default

				elif property_type == 'float':

					value = float_default

				elif property_type =='text':

					value = text_default

				elif property_type =='boolean':

					value = boolean_default

				elif property_type == 'lines':

					value = lines_default

				elif property_type == 'list':

					value = list_default

				elif property_type == 'tokens':

					value = tokens_default

				elif property_type == 'date':

					value = date_default

				elif property_type == 'datetime':

					value = datetime_default

				elif property_type == 'time':

					value = time_default

				else:

					raise SyntaxError, 'Unknown property type %s' % property_type

			exec("self.%s = value" % property['id'])


	def base_after_edit(self, property_sheet=None, REQUEST=None, RESPONSE=None):
		"""Called after an object has been edited."""

		pass


	def base_edit_properties(self, REQUEST, RESPONSE):
		"""Update properties, with a nice redirect."""

		if REQUEST.has_key('property_sheet'):

			property_sheet = REQUEST['property_sheet']

		else:

			property_sheet = None

		changed_properties = []

		# This method assumes that all properties defined
		# are settable by anyone with access to the object.

		if REQUEST.has_key('__ignore_keyerror') and REQUEST['__ignore_keyerror']:

			for property in self._properties:

				try:

					exec("truth = (self.%s != REQUEST['%s'])" % (property['id'], property['id']))

					if truth:

						changed_properties.append((
							property['id'],
							REQUEST[property['id']],
							getattr(self, property['id'])))

					exec("self.%s = REQUEST['%s']" % (property['id'], property['id']))

				except KeyError:

					pass

		else:

			for property in self._properties:

				exec("truth = (self.%s != REQUEST['%s'])" % (property['id'], property['id']))
 
				if truth:

					changed_properties.append((
						property['id'],
						REQUEST[property['id']],
						getattr(self, property['id'])))

				exec("self.%s = REQUEST['%s']" % (property['id'], property['id']))

		if changed_properties:

			self.partial_reindex(map(lambda x: x[0], changed_properties))

			# FIXME
			# Remove this when partial_reindex is done

			self.reindex_object()

			# Workaround for old-style properties

			if hasattr(self.aq_base, 'history'):

				self.history.add_entry(REQUEST['AUTHENTICATED_USER'].getId(), property_sheet, changed_properties)

			self.base_after_edit(property_sheet=property_sheet, REQUEST=REQUEST, RESPONSE=RESPONSE)

		if REQUEST.has_key('__referrer'):

			REQUEST.RESPONSE.redirect(REQUEST['__referrer'])

		else:

			REQUEST.RESPONSE.redirect(self.absolute_url() + '/base_edit')


	def partial_reindex(self, indexes, catalog_name='Catalog'):
		"""Does a partial reindex of the object, based on the indexes specified."""

		pass


	def get_last_modified_by(self):
		"""Returns the username of the user which last
		modified this instance."""

		undoable_transactions = self.undoable_transactions()

		if undoable_transactions:

			return undoable_transactions.pop()['user_name']

		else:

			return self.creator
