###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__doc__ = """Widget tag."""
__version__ = "0.0.1"


from DocumentTemplate import DT_Util, DT_String, VSEval
import html_widgets
import composite_widgets
import types


class widget_tag:
	"""Tag for displaying widgets."""

	name = 'widget'
	expr = None


	def get_type(self):
		"""Returns the type of widget."""

		return self.widget.type


	def __init__(self, arguments):

		parameters = DT_Util.parse_params(arguments, name='', expr='', html_quote='', multiple='',
						options=[], size='', rows='', cols='', selected=[],
						rowspan='', colspan='', onclick='', onchange='', value='',
						true='', false='', separator_html='')

		self.type = parameters['']
		self.html_quote = parameters.has_key('html_quote')

		try:

			self.separator_html = parameters['separator_html']

		except KeyError:

			self.separator_html = ''

		try:

			self.name = parameters['name']

		except KeyError:

			self.name = ''

		try:

			self.size = parameters['size']

		except KeyError:

			self.size = None

		if self.type == 'text':

			self.widget = \
				html_widgets.text(name=self.name, size=self.size)
			self.value_expression = parameters['value']

		elif self.type == 'textarea':

			try:

				self.rows = parameters['rows']
				self.cols = parameters['cols']

			except KeyError:

				try:

					self.rows = parameters['rowspan']
					self.cols = parameters['colspan']

				except KeyError:

					self.rows = None
					self.cols = None

			self.widget = \
				html_widgets.textarea(name=self.name,
					rows=self.rows,
					cols=self.cols)

			self.value_expression = parameters['value']

		elif self.type == 'select':

			self.multiple = parameters.has_key('multiple')

			self.widget = \
				html_widgets.select(name=self.name,
					multiple=self.multiple,
					size=self.size)

			if parameters.has_key('selected'):

				self.selected_parameter = parameters['selected']

			else:

				self.selected_parameter = ""

			self.options_expression = parameters['options']

		elif self.type == 'boolean':

			self.value_expression = parameters['value']

			try:

				self.true = parameters['true']
				self.false = parameters['false']

			except KeyError:

				self.true = None
				self.false = None

			self.widget = \
				composite_widgets.boolean_widget(name=self.name,
					true=self.true, false=self.false)

		elif self.type == 'checkbox_collection':

			self.widget = html_widgets.checkbox_collection(name=self.name)

			if parameters.has_key('selected'):

				self.selected_parameter = parameters['selected']

			else:

				self.selected_parameter = ""

			self.options_expression = parameters['options']

		elif self.type in ('datetime', 'date', 'time'):

			try:

				self.value_expression = parameters['value']

			except KeyError:

				self.value_expression = None

		else:

			raise 'UnknownWidgetType', self.type

		self.parameters = parameters


	def __call__(self, mapping):
		"""Renders the tag."""

		if self.type == 'select':

			options_evaluation = VSEval.Eval(self.options_expression)

			if self.selected_parameter:

				selected_parameter = VSEval.Eval(self.selected_parameter)
				selected_parameter = selected_parameter.eval(mapping)

			else:

				selected_parameter = []

			parameter_options = options_evaluation.eval(mapping)

			self.widget = html_widgets.make_select(name=self.name,
						size=self.size,
						multiple=self.multiple,
						options=parameter_options,
						selected_values=selected_parameter)

		elif self.type == 'checkbox_collection':

			options_evaluation = VSEval.Eval(self.options_expression)

			if self.selected_parameter:

				selected_parameter = VSEval.Eval(self.selected_parameter)
				selected_parameter = selected_parameter.eval(mapping)

			else:

				selected_parameter = []

			parameter_options = options_evaluation.eval(mapping)

			self.widget = html_widgets.make_checkbox_collection(name=self.name,
						options=parameter_options,
						selected_values=selected_parameter)

		elif self.type == 'boolean':

			value = VSEval.Eval(self.value_expression).eval(mapping)
			self.widget = composite_widgets.boolean_widget(name=self.widget.name,
						selected=value, separator_html=self.separator_html)

		elif self.type in ('datetime', 'date', 'time'):

			if self.value_expression:

				value = VSEval.Eval(self.value_expression).eval(mapping)

			else:

				value = None

			if self.type == 'datetime':

				self.widget = composite_widgets.datetime_widget(
						name=self.name, datetime=value,	
						separator_html=self.separator_html)

			elif self.type == 'date':

				self.widget = composite_widgets.date_widget(
						name=self.name, datetime=value,
						separator_html=self.separator_html)

			elif self.type == 'time':

				self.widget = composite_widgets.time_widget(
						name=self.name, seconds=value,
						separator_html=self.separator_html)


		elif self.type == 'text':

			self.widget.value = VSEval.Eval(self.value_expression).eval(mapping)

		elif self.type == 'textarea':

			self.widget.value = VSEval.Eval(self.value_expression).eval(mapping)

		return self.widget(html_quote=self.html_quote)


DT_String.String.commands['widget'] = widget_tag
